@@ -70,7 +70,7 @@ SyntaxDef     PagerDecorationSyntax = {
     {0, "Align", 5, TT_FLAG, PAGER_ALIGN_ID, &AlignSyntax}, \
     {0, "SmallFont", 9, TT_FONT, PAGER_SMALL_FONT_ID, NULL}, \
     {TF_DONT_SPLIT, "ShadeButton", 11, TT_TEXT, PAGER_SHADE_BUTTON_ID, NULL}, \
-    {TF_INDEXED, "Style", 5, TT_TEXT, PAGER_STYLE_ID, NULL}, \
+    {TF_INDEXED|TF_QUOTES_OPTIONAL, "Style", 5, TT_QUOTED_TEXT, PAGER_STYLE_ID, NULL}, \
     {0, "ActiveDeskBevel", 15,   TT_FLAG, PAGER_ActiveBevel_ID   , &BevelSyntax}, \
     {0, "InActiveDeskBevel", 17, TT_FLAG, PAGER_InActiveBevel_ID , &BevelSyntax}, \
     {TF_DONT_REMOVE_COMMENTS, "Decoration", 10, TT_TEXT, PAGER_DECORATION_ID, &PagerDecorationSyntax}
@@ -352,7 +352,7 @@ ParsePagerOptions (const char *filename, char *myname, int desk1, int desk2)
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
 
-    config->balloon_conf = Process_balloonOptions (Storage, NULL);
+    config->balloon_conf = Process_balloonOptions (Storage, NULL, BALLOON_ID_START);
 
 	for (pCurr = Storage; pCurr; pCurr = pCurr->next)
 	{
