@@ -37,6 +37,7 @@
 
 #define WHARF_FOLDER_END 	"~Folder"
 
+
 TermDef       WhevTerms[] = {
 	{TF_NO_MYNAME_PREPENDING | TF_SYNTAX_TERMINATOR, "push", 4, TT_FILENAME, WHEV_PUSH_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING | TF_SYNTAX_TERMINATOR, "close_folder", 12, TT_FILENAME, WHEV_CLOSE_FOLDER_ID, NULL},
@@ -76,11 +77,13 @@ SyntaxDef     WhevSyntax = {
     ASCF_DEFINE_KEYWORD(WHARF, 0, FlipLabel			, TT_FLAG		, NULL), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, FitContents		, TT_FLAG		, NULL), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, ShapeToContents	, TT_FLAG		, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, StretchBackground	, TT_FLAG		, NULL), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, AlignContents		, TT_FLAG		, &AlignSyntax), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, Bevel				, TT_FLAG		, &BevelSyntax), \
 	ASCF_DEFINE_KEYWORD(WHARF, 0, CompositionMethod	, TT_INTEGER	, NULL), \
 	ASCF_DEFINE_KEYWORD(WHARF, 0, FolderOffset		, TT_INTEGER	, NULL), \
-	ASCF_DEFINE_KEYWORD(WHARF, 0, OrthogonalFolderOffset, TT_INTEGER	, NULL)
+	ASCF_DEFINE_KEYWORD(WHARF, 0, OrthogonalFolderOffset, TT_INTEGER	, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, ShowHints			, TT_FLAG		, &BalloonContentsSyntax)
 
 
 #define WHARF_PRIVATE_TERMS \
@@ -101,7 +104,6 @@ TermDef       WharfTerms[] = {
     {TF_OBSOLETE, "NoWithdraw", 10,       TT_FLAG, WHARF_NoWithdraw_ID, NULL},
 /* TextureType, MaxColors, BgColor, TextureColor, and Pixmap are obsolete */
 	{TF_OBSOLETE, "TextureType", 11, TT_UINTEGER, WHARF_TextureType_ID, NULL},
-    {TF_OBSOLETE, "MaxColors", 9,    TT_UINTEGER, WHARF_MaxColors_ID, NULL},
     {TF_OBSOLETE, "BgColor", 7,      TT_COLOR, WHARF_BgColor_ID, NULL},
     {TF_OBSOLETE, "TextureColor", 12,TT_COLOR, WHARF_TextureColor_ID, NULL},
     {TF_OBSOLETE, "Pixmap", 6,       TT_FILENAME, WHARF_Pixmap_ID, NULL},
@@ -169,6 +171,7 @@ flag_options_xref WharfFlags[] = {
     {WHARF_FlipLabel, WHARF_FlipLabel_ID, 0},
     {WHARF_FitContents, WHARF_FitContents_ID, 0},
     {WHARF_ShapeToContents, WHARF_ShapeToContents_ID, 0 },
+    {WHARF_StretchBackground, WHARF_StretchBackground_ID, 0 },
     {0, 0, 0}
 };
 
@@ -193,6 +196,8 @@ DestroyWharfButton (WharfButton **pbtn)
     /* delete members */
 	if (btn->title != NULL)
 		free (btn->title);
+	if (btn->comment != NULL)
+		free (btn->comment);
 
 	if( btn->contents )
 	{
@@ -236,6 +241,7 @@ CreateWharfConfig ()
     config->withdraw_style = WITHDRAW_ON_EDGE_BUTTON_AND_SHOW ;
     
 	config->AlignContents = WHARF_DEFAULT_AlignContents ;
+	config->ShowHints = WHARF_DEFAULT_ShowHints ;
 
 	config->more_stuff = NULL;
 
@@ -349,8 +355,10 @@ PrintWharfConfig(WharfConfig *config )
 	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,FlipLabel);
 	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,FitContents);
 	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,ShapeToContents );
+	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,StretchBackground );
 	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,AlignContents );
 	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,Bevel );
+	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,ShowHints );
 	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,CompositionMethod);
 	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,FolderOffset);
 	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,OrthogonalFolderOffset);
@@ -415,7 +423,7 @@ WharfSpecialFunc (ConfigDef * config)
 		config->cursor = cur;
         /* we are at the beginning of the function definition right now - lets process it :*/
         /* read in entire function definition */
-		GetNextStatement (config, 1);
+		GetNextStatement (config);
         /* lets find us the term for this definition */
         print_trimmed_str( "config->current_data", config->current_data );
         LOCAL_DEBUG_OUT( "curr_data_len = %d", config->current_data_len);
@@ -632,7 +640,7 @@ ParseWharfOptions (const char *filename, char *myname)
 
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
-	config->balloon_conf = Process_balloonOptions (Storage, NULL);
+	config->balloon_conf = Process_balloonOptions (Storage, NULL, BALLOON_ID_START);
 
 	for (pCurr = Storage; pCurr; pCurr = pCurr->next)
 	{
@@ -681,10 +689,6 @@ ParseWharfOptions (const char *filename, char *myname)
 			 set_flags (config->set_flags, WHARF_TEXTURE_TYPE);
 			 config->texture_type = item.data.integer;
 			 break;
-		 case WHARF_MaxColors_ID:
-			 set_flags (config->set_flags, WHARF_MAX_COLORS);
-			 config->max_colors = item.data.integer;
-			 break;
 		 case WHARF_BgColor_ID:
 			 set_string_value (&(config->bg_color), item.data.string, &(config->set_flags), WHARF_BG_COLOR);
 			 break;
@@ -727,6 +731,7 @@ ParseWharfOptions (const char *filename, char *myname)
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,LabelLocation ); 
 		ASCF_HANDLE_ALIGN_KEYWORD_CASE(WHARF,config,pCurr,AlignContents ); 
 		ASCF_HANDLE_BEVEL_KEYWORD_CASE(WHARF,config,pCurr,Bevel); 
+		ASCF_HANDLE_SUBSYNTAX_KEYWORD_CASE(WHARF,config,pCurr,ShowHints,BalloonContents);
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,CompositionMethod ); 
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,FolderOffset ); 
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,OrthogonalFolderOffset ); 
@@ -753,6 +758,7 @@ SHOW_CHECKPOINT;
     return config;
 }
 
+
 #if 0
 
 FreeStorageElem **WharfFolder2FreeStorage (SyntaxDef * syntax, FreeStorageElem ** tail, WharfButton * folder, int root);
@@ -881,9 +887,6 @@ WriteWharfOptions (const char *filename, char *myname, WharfConfig * config, uns
 	/* texture_type  */
 	if (get_flags (config->set_flags, WHARF_TEXTURE_TYPE))
         tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->texture_type, WHARF_TextureType_ID);
-	/* max_colors */
-	if (get_flags (config->set_flags, WHARF_MAX_COLORS))
-        tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->max_colors, WHARF_MaxColors_ID);
 	/* animate_steps */
 	if (get_flags (config->set_flags, WHARF_ANIMATE_STEPS))
         tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->animate_steps, WHARF_AnimateSteps_ID);
