@@ -165,7 +165,13 @@ typedef struct ASImage
 									 * referenced by some other code */
   int                    ref_count ;/* this will tell us what us how many
 									 * times */
-  char                  *name ;     /* readonly copy of image name */
+
+	
+  char                  *name ;     /* readonly copy of image name 
+  									 * this name is a hash value used to 
+									 * store image in the image-man's hash,
+									 * and gets freed automagically on image 
+									 * removal from hash */
 
 #define ASIM_DATA_NOT_USEFUL	(0x01<<0)
 #define ASIM_VECTOR_TOP2BOTTOM	(0x01<<1)
@@ -176,6 +182,7 @@ typedef struct ASImage
 #define ASIM_ALPHA_IS_BITMAP	(0x01<<4) 
 #define ASIM_RGB_IS_BITMAP		(0x01<<5) 
 #define ASIM_XIMAGE_NOT_USEFUL	(0x01<<6)
+#define ASIM_NAME_IS_FILENAME	(0x01<<7)
 
   ASFlagType			 flags ;    /* combination of the above flags */
   
@@ -538,6 +545,7 @@ ASImage *create_asimage( unsigned int width, unsigned int height, unsigned int c
 ASImage *create_static_asimage( unsigned int width, unsigned int height, unsigned int compression);
 ASImage *clone_asimage( ASImage *src, ASFlagType filter );
 void destroy_asimage( ASImage **im );
+Bool asimage_replace (ASImage *im, ASImage *from);
 /****f* libAfterImage/asimage/set_asimage_vector()
  * NAME
  * set_asimage_vector() This function replaces contents of the vector 
@@ -643,6 +651,8 @@ void     destroy_image_manager( struct ASImageManager *imman, Bool reusable );
  * ASImageManager object itself is destroyed.
  *********/
 Bool     store_asimage( ASImageManager* imageman, ASImage *im, const char *name );
+void	 relocate_asimage( ASImageManager* to_imageman, ASImage *im );
+
 /****f* libAfterImage/asimage/fetch_asimage()
  * NAME
  * fetch_asimage()
