@@ -112,6 +112,9 @@ static char* cdata_str = XML_CDATA_STR;
  *              color.
  * 	composite - superimpose arbitrary number of images using one of 15
  *              available methods.
+ *  if        - conditional processing based on value of the variables
+ *  set       - sets value of the variable
+ *  printf    - formated printing of the value of the variable
  *
  * Each tag generates new image as the result of the transformation -
  * existing images are never modified and could be reused as many times
@@ -424,8 +427,8 @@ ASImage *commit_xml_image_built( ASImageXMLState *state, char *id, ASImage *resu
 static void
 translate_tag_size(	const char *width_str, const char *height_str, ASImage *imtmp, ASImage *refimg, int *width_ret, int *height_ret )
 {
-	int width_ref = 1;
-	int height_ref = 1;
+	int width_ref = 0;
+	int height_ref = 0;
 	int width = 0, height = 0 ; 
 	LOCAL_DEBUG_OUT("width_str = \"%s\", height_str = \"%s\", imtmp = %p, refimg = %p", width_str?width_str:"(null)", height_str?height_str:"(null)", imtmp, refimg ); 
 	
@@ -862,7 +865,7 @@ handle_asxml_tag_composite( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t*
  * image to fit the location of the resulting window, if one is displayed.
  ******/
 static ASImage *
-handle_asxml_tag_img( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm)
+handle_asxml_tag_img( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm, int dst_width, int dst_height)
 {
 	ASImage *result = NULL ;
 	const char* src = NULL;
@@ -878,10 +881,24 @@ handle_asxml_tag_img( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm)
 		if (rp) {
 			get_drawable_size(rp, &width, &height);
 			result = pixmap2asimage(state->asv, rp, 0, 0, width, height, 0xFFFFFFFF, False, 100);
+			if( dst_width == 0 ) dst_width = width ; 
+			if( dst_height == 0 ) dst_height = height ; 
+			if( dst_width != width || dst_height != height ) 
+			{
+				ASImage *tmp = scale_asimage( NULL, result, dst_width, dst_height, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
+				if( tmp ) 
+				{
+					safe_asimage_destroy( result );
+					result = tmp ;
+				}  	
+			}	
 		}
 	} else if (src) {
-		show_progress("Loading image [%s] using imman (%p) with search path \"%s\".", src, state->imman, state->imman?state->imman->search_path[0]:"");
-		result = get_asimage( state->imman, src, 0xFFFFFFFF, 100 );
+		show_progress("Loading image [%s] using imman (%p) with search path \"%s\" (dst_size = %dx%d).", src, state->imman, state->imman?state->imman->search_path[0]:"", dst_width, dst_height);
+		if( dst_width != 0 || dst_height != 0 ) 
+			result = get_thumbnail_asimage( state->imman, src, dst_width, dst_height, (dst_width==0||dst_height==0)?AS_THUMBNAIL_PROPORTIONAL:0 );
+		else
+			result = get_asimage( state->imman, src, 0xFFFFFFFF, 100 );
 	}
 	return result;
 }	
@@ -890,7 +907,7 @@ handle_asxml_tag_img( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm)
  * NAME
  * recall - recall previously generated and named image by its id.
  * SYNOPSIS
- * <recall id="new_id" srcid="image_id">
+ * <recall id="new_id" srcid="image_id" default_src="filename"/>
  * ATTRIBUTES
  * id       Optional.  Image will be given this name for future reference.
  * srcid    Required.  An image ID defined with the "id" parameter for
@@ -900,17 +917,27 @@ static ASImage *
 handle_asxml_tag_recall( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm)
 {
 	ASImage *result = NULL ;
+	xml_elem_t* ptr = parm ; 
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p", doc, parm ); 
-	while ( parm && !result ) 
+	while ( ptr && !result ) 
 	{	
-		if (!strcmp(parm->tag, "srcid"))
+		if (!strcmp(ptr->tag, "srcid"))
 		{ 
-			show_progress("Recalling image id [%s] from imman %p.", parm->parm, state->imman);
-			result = fetch_asimage(state->imman, parm->parm );
+			show_progress("Recalling image id [%s] from imman %p.", ptr->parm, state->imman);
+			result = fetch_asimage(state->imman, ptr->parm );
 			if (!result)
-				show_error("Image recall failed for id [%s].", parm->parm);
+				show_warning("Image recall failed for id [%s].", parm->parm);
 		}	
-		parm = parm->next ;
+		ptr = ptr->next ;
+	}
+	if( result == NULL ) 
+	{
+		for( ptr = parm ; ptr && !result ; ptr = ptr->next )
+			if (!strcmp(ptr->tag, "default_src"))
+			{ 
+				show_progress("loading default image [%s] from imman %p.", ptr->parm, state->imman);
+				result = get_asimage( state->imman, ptr->parm, 0xFFFFFFFF, 100 );
+			}
 	}
 	return result;
 }	
@@ -1020,7 +1047,7 @@ handle_asxml_tag_color( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* par
  * NAME
  * printf - prints variable value to standard output.
  * SYNOPSIS
- * <printf format="format_string" var="variable_name" val="value"/>
+ * <printf format="format_string" var="variable_name" val="expression"/>
  * ATTRIBUTES
  * format_string  Standard C format string with exactly 1 placeholder.
  * var            Name of the variable, which value will be printed.
@@ -1068,6 +1095,116 @@ handle_asxml_tag_printf( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* pa
 		
 	return NULL;
 }
+/****** libAfterImage/asimagexml/set
+ * NAME
+ * set - declares variable, assigning it a numeric value of expression.
+ * SYNOPSIS
+ * <set var="variable_name" domain="var_domain" val="expression"/>
+ * ATTRIBUTES
+ * var            Name of the variable, which value will be set.
+ * val            math expression to be evaluated.
+ * domain         (optional) variable's domain to be prepended to its name
+ *                using format var_domain.variable_name
+ ******/
+static ASImage *
+handle_asxml_tag_set( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm)
+{
+	xml_elem_t* ptr ;
+	const char* var_domain = NULL ;
+	const char* var = NULL;
+	int val = 0 ;
+	LOCAL_DEBUG_OUT("doc = %p, parm = %p", doc, parm ); 
+	for (ptr = parm ; ptr ; ptr = ptr->next) 
+	{
+		if (!strcmp(ptr->tag, "var")) 			var = ptr->parm;
+		else if (!strcmp(ptr->tag, "domain")) 	var_domain = ptr->parm;
+		else if (!strcmp(ptr->tag, "val"))  	val = (int)parse_math(ptr->parm, NULL, 0);
+	}
+   		
+	if( var != NULL ) 
+	{	
+		char *tmp = (char*)var ; 
+		if( var_domain && var_domain[0] != '\0' )
+		{
+			int vd_len = var_domain?strlen(var_domain):0 ;
+			tmp = safemalloc( vd_len + 1 + strlen(var) + 1 );
+			sprintf( tmp, ( var_domain[vd_len-1] != '.' )?"%s.%s":"%s%s", var_domain, var );
+		}
+		asxml_var_insert( tmp, val );
+		if( tmp != var ) 
+			free( tmp );
+	}
+		
+	return NULL;
+}
+/****** libAfterImage/asimagexml/if
+ * NAME
+ * if - evaluates logical expression and if result evaluates to not true(or false 
+ * if <unless> tag is used ), handles tags within.
+ * SYNOPSIS
+ * <if val1="expression" [op="gt|lt|ge|le|eq|ne" val2="expression"]/>
+ * 	[<then>...</then><else>...</else>]
+ * </if>
+ * <unless val1="expression" [op="gt|lt|ge|le|eq|ne" val2="expression"]/>
+ * ATTRIBUTES
+ * val1            math expression to be evaluated.
+ * val2            math expression to be evaluated.
+ * op         	 (optional) comparison op to be applied to values
+ * EXAMPLE
+ * <if val1="$ascs.Base.value" val2="50" op="gt"><then>...</then><else>...</else></if>
+ ******/
+static ASImage *
+handle_asxml_tag_if( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm)
+{
+	xml_elem_t* ptr ;
+	int val1 = 0, val2 = 0 ;
+	const char *op = NULL ;
+	int res = 0 ; 
+	ASImage *im = NULL, *imtmp = NULL  ; 
+	LOCAL_DEBUG_OUT("doc = %p, parm = %p", doc, parm ); 
+	
+	for (ptr = parm ; ptr ; ptr = ptr->next) 
+	{
+		if (!strcmp(ptr->tag, "op")) 			op = ptr->parm;
+		else if (!strcmp(ptr->tag, "val1"))  	val1 = (int)parse_math(ptr->parm, NULL, 0);
+		else if (!strcmp(ptr->tag, "val2"))  	val2 = (int)parse_math(ptr->parm, NULL, 0);
+	}
+   		
+	if( op != NULL ) 
+	{	
+		if     ( strcmp(op, "gt") == 0 ) res = (val1 > val2);
+		else if( strcmp(op, "lt") == 0 ) res = (val1 < val2);
+		else if( strcmp(op, "ge") == 0 ) res = (val1 >= val2);
+		else if( strcmp(op, "le") == 0 ) res = (val1 <= val2);
+		else if( strcmp(op, "eq") == 0 ) res = (val1 == val2);
+		else if( strcmp(op, "ne") == 0 ) res = (val1 != val2);
+	}
+
+	if( doc->tag[0] == 'u' ) /* <unless> */
+		res = !res ;
+
+	ptr = NULL ;
+	for (ptr = doc->child ; ptr ; ptr = ptr->next) 
+	{
+		if( strcmp(ptr->tag, res?"then":"else" ) )
+		{
+			ptr = ptr->child ;
+			break;
+		}
+		if( res && ptr->next == NULL ) 
+			ptr = doc->child ;
+	}
+	
+	while( ptr ) 
+	{
+		imtmp = build_image_from_xml(state->asv, state->imman, state->fontman, ptr, NULL, state->flags, state->verbose, state->display_win);
+		if( im && imtmp ) safe_asimage_destroy( im ); 
+		if( imtmp ) im = imtmp ;
+		 ptr = ptr->next ;
+	}
+	return im ;
+}
+
 /****** libAfterImage/asimagexml/gradient
  * NAME
  * gradient - render multipoint gradient.
@@ -1953,7 +2090,6 @@ handle_asxml_tag_hsv( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm,
 		hue_offset = 0 ; 
 		saturation_offset = -99 ;
 	}
-	show_progress("adjusting HSV of the image by [%d,%d,%d] affected hues are %+d-%+d.result = %p", hue_offset, saturation_offset, value_offset, affected_hue-affected_radius, affected_hue+affected_radius, result);
 	if (hue_offset!=0 || saturation_offset != 0 || value_offset != 0 ) 
 	{
 		result = adjust_asimage_hsv(state->asv, imtmp, xorig, yorig, width, height,
@@ -1961,6 +2097,7 @@ handle_asxml_tag_hsv( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm,
 									hue_offset, saturation_offset, value_offset,
 				                    ASA_ASImage, 100, ASIMAGE_QUALITY_TOP);
 	}
+	show_progress("adjusted HSV of the image by [%d,%d,%d] affected hues are %+d-%+d.result = %p", hue_offset, saturation_offset, value_offset, affected_hue-affected_radius, affected_hue+affected_radius, result);
 	return result;
 }
 
@@ -2058,8 +2195,10 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 		else if (!strcmp(doc->tag, "text")) 
 			result = handle_asxml_tag_text( &state, doc, parm );  	
 		else if (!strcmp(doc->tag, "img")) 
-			result = handle_asxml_tag_img( &state, doc, parm );
-		else if (!strcmp(doc->tag, "recall")) 
+		{
+			translate_tag_size(	width_str, height_str, NULL, refimg, &width, &height );  
+			result = handle_asxml_tag_img( &state, doc, parm, width, height );
+		}else if (!strcmp(doc->tag, "recall")) 
 			result = handle_asxml_tag_recall( &state, doc, parm );
 		else if (!strcmp(doc->tag, "release"))
 			result = handle_asxml_tag_release( &state, doc, parm );
@@ -2067,6 +2206,10 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 			result = handle_asxml_tag_color( &state, doc, parm ); 
 		else if (!strcmp(doc->tag, "printf"))
 			result = handle_asxml_tag_printf( &state, doc, parm ); 
+		else if (!strcmp(doc->tag, "set"))
+			result = handle_asxml_tag_set( &state, doc, parm ); 
+		else if (!strcmp(doc->tag, "if") || !strcmp(doc->tag, "unless") )
+			result = handle_asxml_tag_if( &state, doc, parm ); 
 		else if ( !strcmp(doc->tag, "gradient") )
 		{	
 			translate_tag_size(	width_str, height_str, NULL, refimg, &width, &height );  
@@ -2080,7 +2223,7 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 		}else
 		{	
 			ASImage *imtmp = NULL ; 
-	
+
 			for (ptr = doc->child ; ptr && !imtmp ; ptr = ptr->next) 
 				imtmp = build_image_from_xml(asv, imman, fontman, ptr, NULL, flags, verbose, display_win);
 
