@@ -27,6 +27,7 @@
 #else
 #include "config.h"
 #endif
+//#undef HAVE_MMX
 
 #ifdef DO_CLOCKING
 #if TIME_WITH_SYS_TIME
@@ -51,6 +52,9 @@
 #endif
 #include <math.h>
 
+#ifdef HAVE_MMX
+#include <mmintrin.h>
+#endif
 
 #ifdef _WIN32
 # include "win32/afterbase.h"
@@ -63,6 +67,9 @@
 #include "imencdec.h"
 #include "transform.h"
 
+ASVisual __transform_fake_asv = {0};
+
+
 /* ******************************************************************************/
 /* below goes all kinds of funky stuff we can do with scanlines : 			   */
 /* ******************************************************************************/
@@ -315,11 +322,21 @@ reverse_component( register CARD32 *src, register CARD32 *dst, int *unused, int
 static inline void
 add_component( CARD32 *src, CARD32 *incr, int *scales, int len )
 {
-	int i = 0;
-
 	len += len&0x01;
+#ifdef HAVE_MMX   
 #if 1
-#ifdef HAVE_MMX
+	if( asimage_use_mmx )
+	{
+		int i = 0;
+		__m64  *vdst = (__m64*)&(src[0]);
+		__m64  *vinc = (__m64*)&(incr[0]);
+		len = len>>1;
+		do{
+			vdst[i] = _mm_add_pi32(vdst[i],vinc[i]);  /* paddd */
+		}while( ++i  < len );
+		_mm_empty();
+	}else
+#else
 	if( asimage_use_mmx )
 	{
 		double *ddst = (double*)&(src[0]);
@@ -340,6 +357,7 @@ add_component( CARD32 *src, CARD32 *incr, int *scales, int len )
 #endif
 	{
 		register int c1, c2;
+		int i = 0;
 		do{
 			c1 = (int)src[i] + (int)incr[i] ;
 			c2 = (int)src[i+1] + (int)incr[i+1] ;
@@ -361,7 +379,7 @@ rbitshift_component( register CARD32 *src, register CARD32 *dst, int shift, int
 #endif
 
 static inline void
-start_component_interpolation( CARD32 *c1, CARD32 *c2, CARD32 *c3, CARD32 *c4, register CARD32 *T, register CARD32 *step, CARD16 S, int len)
+start_component_interpolation( CARD32 *c1, CARD32 *c2, CARD32 *c3, CARD32 *c4, register CARD32 *T, register CARD32 *step, int S, int len)
 {
 	register int i;
 	for( i = 0 ; i < len ; i++ )
@@ -772,7 +790,7 @@ scale_image_up( ASImageDecoder *imdec, ASImageOutput *imout, int h_ratio, int *s
                     imout->output_image_scanline( imout, c1, 1);
                 }else
                 {
-                    SCANLINE_COMBINE(start_component_interpolation,*c1,*c2,*c3,*c4,*c1,step,(CARD8)S,out_width);
+                    SCANLINE_COMBINE(start_component_interpolation,*c1,*c2,*c3,*c4,*c1,step,S,out_width);
                     do
                     {
                         imout->output_image_scanline( imout, c1, 1);
@@ -844,7 +862,9 @@ scale_asimage( ASVisual *asv, ASImage *src, int to_width, int to_height,
 	int h_ratio ;
 	int *scales_h = NULL, *scales_v = NULL;
 	START_TIME(started);
-
+	
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+	
 	if( !check_scale_parameters(src,src->width, src->height,&to_width,&to_height) )
 		return NULL;
 	if( (imdec = start_image_decoding(asv, src, SCL_DO_ALL, 0, 0, 0, 0, NULL)) == NULL )
@@ -881,9 +901,6 @@ scale_asimage( ASVisual *asv, ASImage *src, int to_width, int to_height,
 		fprintf( stderr, " %d", scales_v[i] );
 	  fprintf( stderr, "\n" );
 	}
-#endif
-#ifdef HAVE_MMX
-	mmx_init();
 #endif
 	if((imout = start_image_output( asv, dst, out_format, QUANT_ERR_BITS, quality )) == NULL )
 	{
@@ -898,9 +915,6 @@ scale_asimage( ASVisual *asv, ASImage *src, int to_width, int to_height,
 			scale_image_up( imdec, imout, h_ratio, scales_h, scales_v );
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	free( scales_h );
 	free( scales_v );
 	stop_image_decoding( &imdec );
@@ -924,6 +938,9 @@ scale_asimage2( ASVisual *asv, ASImage *src,
 
 	if( src == NULL ) 
 		return NULL;
+
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	if( clip_width == 0 )
 		clip_width = src->width ;
 	if( clip_height == 0 )
@@ -964,9 +981,6 @@ scale_asimage2( ASVisual *asv, ASImage *src,
 		fprintf( stderr, " %d", scales_v[i] );
 	  fprintf( stderr, "\n" );
 	}
-#endif
-#ifdef HAVE_MMX
-	mmx_init();
 #endif
 	if((imout = start_image_output( asv, dst, out_format, QUANT_ERR_BITS, quality )) == NULL )
 	{
@@ -981,9 +995,6 @@ scale_asimage2( ASVisual *asv, ASImage *src,
 			scale_image_up( imdec, imout, h_ratio, scales_h, scales_v );
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	free( scales_h );
 	free( scales_v );
 	stop_image_decoding( &imdec );
@@ -1004,6 +1015,8 @@ tile_asimage( ASVisual *asv, ASImage *src,
 	ASImageOutput  *imout ;
 	START_TIME(started);
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 LOCAL_DEBUG_CALLER_OUT( "src = %p, offset_x = %d, offset_y = %d, to_width = %d, to_height = %d, tint = #%8.8lX", src, offset_x, offset_y, to_width, to_height, tint );
 	if( src== NULL || (imdec = start_image_decoding(asv, src, SCL_DO_ALL, offset_x, offset_y, to_width, 0, NULL)) == NULL )
 	{
@@ -1013,9 +1026,6 @@ LOCAL_DEBUG_CALLER_OUT( "src = %p, offset_x = %d, offset_y = %d, to_width = %d,
 
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color );
 
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
 	if((imout = start_image_output( asv, dst, out_format, (tint!=0)?8:0, quality)) == NULL )
 	{
 		LOCAL_DEBUG_OUT( "failed to start image output%s", "");
@@ -1048,9 +1058,6 @@ LOCAL_DEBUG_OUT("tiling actually...%s", "");
 			}
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	stop_image_decoding( &imdec );
 
 	SHOW_TIME("", started);
@@ -1075,13 +1082,15 @@ merge_layers( ASVisual *asv,
 
 LOCAL_DEBUG_CALLER_OUT( "dst_width = %d, dst_height = %d", dst_width, dst_height );
 	
-	prepare_scanline( dst_width, QUANT_ERR_BITS, &dst_line, asv->BGR_mode );
-	dst_line.flags = SCL_DO_ALL ;
-
 	dst = create_destination_image( dst_width, dst_height, out_format, compression_out, ARGB32_DEFAULT_BACK_COLOR );
 	if( dst == NULL )
 		return NULL;
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
+	prepare_scanline( dst_width, QUANT_ERR_BITS, &dst_line, asv->BGR_mode );
+	dst_line.flags = SCL_DO_ALL ;
+
 	imdecs = safecalloc( count+20, sizeof(ASImageDecoder*));
 
 /*  don't really know why the hell we need that ???
@@ -1114,9 +1123,6 @@ LOCAL_DEBUG_CALLER_OUT( "dst_width = %d, dst_height = %d", dst_width, dst_height
 	}
 	if( i < count )
 		count = i+1 ;
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
 
 	if(imdecs[0] == NULL || (imout = start_image_output( asv, dst, out_format, QUANT_ERR_BITS, quality)) == NULL )
 	{
@@ -1125,6 +1131,7 @@ LOCAL_DEBUG_CALLER_OUT( "dst_width = %d, dst_height = %d", dst_width, dst_height
 				stop_image_decoding( &(imdecs[i]) );
 
         destroy_asimage( &dst );
+		free_scanline( &dst_line, True );
     }else
 	{
 		int y, max_y = 0;
@@ -1207,9 +1214,6 @@ LOCAL_DEBUG_OUT( "min_y = %d, max_y = %d", min_y, max_y );
 			imout->output_image_scanline( imout, &dst_line, 1);
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	for( i = 0 ; i < count ; i++ )
 		if( imdecs[i] != NULL )
 		{
@@ -1449,8 +1453,11 @@ make_gradient( ASVisual *asv, ASGradient *grad,
 	int line_len = width;
 	START_TIME(started);
 LOCAL_DEBUG_CALLER_OUT( "type = 0x%X, width=%d, height = %d, filter = 0x%lX", grad->type, width, height, filter );
-	if( asv == NULL || grad == NULL )
+	if( grad == NULL )
 		return NULL;
+
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	if( width == 0 )
 		width = 2;
  	if( height == 0 )
@@ -1533,9 +1540,8 @@ LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height
 	filter = get_asimage_chanmask(src);
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
 
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	if((imout = start_image_output( asv, dst, out_format, 0, quality)) == NULL )
 	{
         destroy_asimage( &dst );
@@ -1626,9 +1632,6 @@ LOCAL_DEBUG_OUT("flip-flopping actually...%s", "");
 		free_scanline( &result, True );
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	SHOW_TIME("", started);
 	return dst;
 }
@@ -1648,9 +1651,8 @@ mirror_asimage( ASVisual *asv, ASImage *src,
 	LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height = %d", offset_x, offset_y, to_width, to_height );
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
 
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	if((imout = start_image_output( asv, dst, out_format, 0, quality)) == NULL )
 	{
         destroy_asimage( &dst );
@@ -1690,9 +1692,6 @@ LOCAL_DEBUG_OUT("miroring actually...%s", "");
 			free_scanline( &result, True );
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	SHOW_TIME("", started);
 	return dst;
 }
@@ -1718,6 +1717,8 @@ LOCAL_DEBUG_CALLER_OUT( "dst_x = %d, dst_y = %d, to_width = %d, to_height = %d",
 	if( to_width == src->width && to_height == src->height && dst_x == 0 && dst_y == 0 )
 		return clone_asimage( src, SCL_DO_ALL );
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
 
 	clip_width = src->width ;
@@ -1736,9 +1737,6 @@ LOCAL_DEBUG_CALLER_OUT( "dst_x = %d, dst_y = %d, to_width = %d, to_height = %d",
 		return dst ;
 	}
 
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
 	if((imout = start_image_output( asv, dst, out_format, 0, quality)) == NULL )
 	{
         destroy_asimage( &dst );
@@ -1820,9 +1818,6 @@ LOCAL_DEBUG_OUT( "filling %d lines with %8.8lX at the end", to_height-(start_y+c
 		}
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	SHOW_TIME("", started);
 	return dst;
 }
@@ -1838,7 +1833,9 @@ Bool fill_asimage( ASVisual *asv, ASImage *im,
 	ASImageDecoder *imdec;
 	START_TIME(started);
 
-	if( asv == NULL || im == NULL )
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
+	if( im == NULL )
 		return False;
 	if( x < 0 )
 	{	width += x ; x = 0 ; }
@@ -1918,6 +1915,8 @@ colorize_asimage_vector( ASVisual *asv, ASImage *im,
 		return False;
 	vector = im->alt.vector ;
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	if((imout = start_image_output( asv, im, out_format, QUANT_ERR_BITS, quality)) == NULL )
 		return False;
 	/* as per ROOT ppl request double data goes from bottom to top,
@@ -2020,6 +2019,8 @@ create_asimage_from_vector( ASVisual *asv, double *vector,
 {
 	ASImage *im = NULL;
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	if( vector != NULL )
 	{
 		im = create_destination_image( width, height, out_format, compression, ARGB32_DEFAULT_BACK_COLOR);
@@ -2074,28 +2075,21 @@ ASImage* blur_asimage_gauss(ASVisual* asv, ASImage* src, double horz, double ver
 
 	if (!src) return NULL;
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 	dst = create_destination_image( src->width, src->height, out_format, compression_out, src->back_color);
 
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
 
 	imout = start_image_output(asv, dst, out_format, 0, quality);
     if (!imout)
     {
         destroy_asimage( &dst );
-#ifdef HAVE_MMX
-		mmx_off();
-#endif
 		return NULL;
 	}
 
 	imdec = start_image_decoding(asv, src, SCL_DO_ALL, 0, 0, dst->width, dst->height, NULL);
 	if (!imdec) {
         destroy_asimage( &dst );
-#ifdef HAVE_MMX
-		mmx_off();
-#endif
 		return NULL;
 	}
 
@@ -2142,10 +2136,6 @@ ASImage* blur_asimage_gauss(ASVisual* asv, ASImage* src, double horz, double ver
 
 	free(gauss);
 
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
-
 	return dst;
 }
 
@@ -2184,6 +2174,8 @@ adjust_asimage_hsv( ASVisual *asv, ASImage *src,
 	ASImageOutput  *imout ;
 	START_TIME(started);
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height = %d, hue = %u", offset_x, offset_y, to_width, to_height, affected_hue );
 	if( src == NULL ) 
 		return NULL;
@@ -2191,9 +2183,6 @@ LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height
 		return NULL;
 
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
 	set_decoder_shift(imdec,8);
 	if((imout = start_image_output( asv, dst, out_format, 8, quality)) == NULL )
 	{
@@ -2202,9 +2191,12 @@ LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height
 	{
 	    CARD32 from_hue1 = 0, from_hue2 = 0, to_hue1 = 0, to_hue2 = 0 ;
 		int y, max_y = to_height;
+		Bool do_greyscale = False ; 
 
 		affected_hue = normalize_degrees_val( affected_hue );
 		affected_radius = normalize_degrees_val( affected_radius );
+		if( value_offset != 0 )
+			do_greyscale = (affected_hue+affected_radius >= 360 || affected_hue-affected_radius <= 0 );
 		if( affected_hue > affected_radius )
 		{
 			from_hue1 = degrees2hue16(affected_hue-affected_radius);
@@ -2273,6 +2265,10 @@ LOCAL_DEBUG_OUT("adjusting actually...%s", "");
 						fprintf( stderr, "OUT %d: rgb = #%4.4lX.%4.4lX.%4.4lX hue = %ld(%ld)     sat = %ld val = %ld\n", __LINE__, r[x], g[x], b[x], h, ((h>>8)*360)>>8, s, v );
 #endif
 					}
+				}else if( do_greyscale ) 
+				{
+					int tmp = (int)r[x] + value_offset ; 
+					g[x] = b[x] = r[x] = (tmp < 0)?0:((tmp>0x00FFFF)?0x00FFff:tmp);
 				}
 			}
 			imdec->buffer.flags = 0xFFFFFFFF ;
@@ -2280,9 +2276,6 @@ LOCAL_DEBUG_OUT("adjusting actually...%s", "");
 		}
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	stop_image_decoding( &imdec );
 
 	SHOW_TIME("", started);
@@ -2371,6 +2364,8 @@ slice_asimage2( ASVisual *asv, ASImage *src,
 	ASImageOutput  *imout = NULL ;
 	START_TIME(started);
 
+	if( asv == NULL ) 	asv = &__transform_fake_asv ;
+
 LOCAL_DEBUG_CALLER_OUT( "sx1 = %d, sx2 = %d, sy1 = %d, sy2 = %d, to_width = %d, to_height = %d", slice_x_start, slice_x_end, slice_y_start, slice_y_end, to_width, to_height );
 	if( src == NULL )
 		return NULL;
@@ -2390,9 +2385,6 @@ LOCAL_DEBUG_CALLER_OUT( "sx1 = %d, sx2 = %d, sy1 = %d, sy2 = %d, to_width = %d,
 		slice_y_start = (slice_y_end > 0 ) ? slice_y_end-1 : 0 ;
 
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
-#ifdef HAVE_MMX
-	mmx_init();
-#endif
 	if((imout = start_image_output( asv, dst, out_format, 0, quality)) == NULL )
 	{
         destroy_asimage( &dst );
@@ -2553,11 +2545,9 @@ LOCAL_DEBUG_CALLER_OUT( "sx1 = %d, sx2 = %d, sy1 = %d, sy2 = %d, to_width = %d,
 				imout->output_image_scanline( imout, out_buf, 1);
 			}
 		}
+		free_scanline( out_buf, False );
 		stop_image_output( &imout );
 	}
-#ifdef HAVE_MMX
-	mmx_off();
-#endif
 	stop_image_decoding( &imdec );
 
 	SHOW_TIME("", started);
