@@ -98,7 +98,7 @@ struct ASCategoryTree *StandardCategories = NULL ;
 struct ASCategoryTree *AfterStepCategories = NULL ;
 struct ASCategoryTree *KDECategories = NULL ;
 struct ASCategoryTree *GNOMECategories = NULL ;
-struct ASCategoryTree *SystemCategories = NULL ;
+struct ASCategoryTree *OtherCategories = NULL ;
 struct ASCategoryTree *CombinedCategories = NULL ;
 
 
@@ -134,9 +134,11 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
 	FUNC_TERM2 (NEED_NAME, "PopUp", F_POPUP),	/* PopUp    "popup_name" [popup_name] */
 	FUNC_TERM2 (NEED_NAME, "Function", F_FUNCTION),	/* Function "function_name" [function_name] */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Category", F_CATEGORY),	/* Category "function_name" category_name */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "CategoryTree", F_CATEGORY_TREE),	/* CategoryTree "function_name" category_name */
 	FUNC_TERM ("MiniPixmap", F_MINIPIXMAP),	/* MiniPixmap "name" */
 	FUNC_TERM ("SmallMiniPixmap", F_SMALL_MINIPIXMAP),	/* SmallMiniPixmap "name" */
 	FUNC_TERM ("LargeMiniPixmap", F_SMALL_MINIPIXMAP),	/* LargeMiniPixmap "name" */
+	FUNC_TERM2 (NEED_NAME, "DesktopEntry", F_DesktopEntry),	/* DesktopEntry "name" */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Exec", F_EXEC),	/* Exec   "name" command */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Module", F_MODULE),	/* Module "name" command */
     FUNC_TERM2 (NEED_NAME | NEED_CMD, "ExecInTerm", F_ExecInTerm),   /* ExecInTerm   "name" command */
@@ -167,8 +169,11 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
 	FUNC_TERM2 (TF_SYNTAX_TERMINATOR, "EndFunction", F_ENDFUNC),
 	FUNC_TERM2 (TF_SYNTAX_TERMINATOR, "EndPopup", F_ENDPOPUP),
 	FUNC_TERM ("TakeScreenShot", F_TAKE_SCREENSHOT),
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Test", F_Test),
 
+	FUNC_TERM2 (NEED_CMD, "Set", F_SET),	/* Set "name" <variable>=<value> */
+
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Test", F_Test),
+	
 	/* this functions require window as aparameter */
 	FUNC_TERM ("&nonsense&", F_WINDOW_FUNC_START),	/* not really a command */
 	FUNC_TERM2 (USES_NUMVALS, "Move", F_MOVE),	/* Move     ["name"] [whereX whereY] */
@@ -566,6 +571,7 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 	}
 
     as_app_args.mask = opt_mask ;
+	as_app_args.gravity = ForgetGravity ;
 #ifndef NO_DEBUG_OUTPUT
     as_app_args.verbosity_level = OUTPUT_VERBOSE_THRESHOLD ;
 #else
@@ -617,7 +623,8 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 
     if (FuncSyntax.term_hash == NULL)
 		PrepareSyntax (&FuncSyntax);
-
+	if( as_app_args.locale == NULL ) 
+		as_app_args.locale = mystrdup("");
 	if( as_app_args.locale && strlen(as_app_args.locale) > 0)
 	{
 		as_set_charset( parse_charset_name( as_app_args.locale ));
@@ -639,14 +646,16 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
     trace_enable_function(as_app_args.trace_calls);
 #endif
 
-	asxml_var_insert("icon.button.width", 64);
-	asxml_var_insert("icon.button.height", 64);
-	asxml_var_insert("icon.width", 48);
-	asxml_var_insert("icon.height", 48);
-	asxml_var_insert("minipixmap.width", 24);
-	asxml_var_insert("minipixmap.height", 24);
-	asxml_var_insert("title.font.size", 14);
-	asxml_var_insert("menu.font.size", 16);
+	asxml_var_insert(ASXMLVAR_IconButtonWidth, 64);
+	asxml_var_insert(ASXMLVAR_IconButtonHeight, 64);
+	asxml_var_insert(ASXMLVAR_IconWidth, 48);
+	asxml_var_insert(ASXMLVAR_IconHeight, 48);
+	asxml_var_insert(ASXMLVAR_MinipixmapWidth, 24);
+	asxml_var_insert(ASXMLVAR_MinipixmapHeight, 24);
+	asxml_var_insert(ASXMLVAR_TitleFontSize, 14);
+	asxml_var_insert(ASXMLVAR_MenuFontSize, 16);
+	asxml_var_insert(ASXMLVAR_MenuShowMinipixmaps, 1);
+	asxml_var_insert(ASXMLVAR_MenuShowUnavailable, 1);
 
 }
 
@@ -814,8 +823,9 @@ make_default_environment()
 	for( i = 0 ; i < ASTool_Count ; ++i )
 		e->tool_command[i] = as_get_default_tool(i);	 
 
-	e->gtkrc_path = make_session_rc_file(Session, GTKRC_FILE);
-  	e->gtkrc20_path = make_session_rc_file(Session, GTKRC20_FILE) ;
+	/* by default - don't do overwrite gtkrc files so to not aggrave people */
+	e->gtkrc_path = NULL ;  /* make_session_rc_file(Session, GTKRC_FILE); */
+  	e->gtkrc20_path = NULL ; /* make_session_rc_file(Session, GTKRC20_FILE) ; */
 
 	return e;
 }
@@ -866,8 +876,8 @@ destroy_asdatabase()
     destroy_user_database();
 }
 
-ASDesktopCategory *
-name2desktop_category( const char *name, ASCategoryTree **tree_return ) 
+static ASCategoryTree*
+name2desktop_category_tree( const char *name, int *tree_name_len ) 
 {
 	ASCategoryTree *ct = CombinedCategories ; 
 	int offset = 0 ;
@@ -885,20 +895,44 @@ name2desktop_category( const char *name, ASCategoryTree **tree_return )
 	{	
 		ct = GNOMECategories ; 
 		offset = 6 ; 
-	}else if( !mystrncasecmp (name, "SYSTEM:", 7) )
+	}else if( !mystrncasecmp (name, "OTHER:", 7) )
 	{	
-		ct = SystemCategories ;
+		ct = OtherCategories ;
 		offset = 7 ; 
 	}else if( !mystrncasecmp (name, "COMBINED:", 9) )
 	{	
 		ct = CombinedCategories; 
 		offset = 9 ;
 	}
+	if( tree_name_len ) 
+		*tree_name_len = offset ;
+	return ct;
+}
+
+ASDesktopCategory *
+name2desktop_category( const char *name, ASCategoryTree **tree_return ) 
+{
+	int offset = 0 ;
+	ASCategoryTree *ct = name2desktop_category_tree( name, &offset );  
+
 	if( tree_return ) 
 		*tree_return = ct ;
+
 	return fetch_desktop_category( ct, name+offset );
 }
 
+ASDesktopEntry *
+name2desktop_entry( const char *name, ASCategoryTree **tree_return ) 
+{
+	int offset = 0 ;
+	ASCategoryTree *ct = name2desktop_category_tree( name, &offset );  
+
+	if( tree_return ) 
+		*tree_return = ct ;
+
+	return fetch_desktop_entry( ct, name+offset );
+}
+
 void
 InitSession()
 {
@@ -932,7 +966,7 @@ free_as_app_args()
 void
 FreeMyAppResources()
 {
-    balloon_init (True);
+	cleanup_default_balloons();
 	destroy_asdatabase();
     mystyle_destroy_all();
     mylook_init( &(ASDefaultScr->Look), True, ASFLAGS_EVERYTHING );
