@@ -484,7 +484,7 @@ file2xml_tree(const char *filename, char *myname, SyntaxDef *syntax )
 		tree->child = xml_elem_new(); 
 		tree->child->tag = mystrdup(syntax->display_name);
 		//c_tag->tag_id = pterm->id ;
-		config2tree_storage(config_reader, &model, False);	   
+		config2tree_storage(config_reader, &model);	   
 		tree->child->child = (xml_elem_t*)model ;
 		DestroyConfig (config_reader);   
 	}
