@@ -128,25 +128,48 @@ void
 get_Xinerama_rectangles (ScreenInfo * scr)
 {
 #ifdef HAVE_XINERAMA
-	register int  i;
-	XineramaScreenInfo *s;
-
-	if ((s = XineramaQueryScreens (dpy, &(scr->xinerama_screens_num))) != NULL)
+	if (XineramaQueryExtension (dpy, &(scr->XineEventBase), &(scr->XineErrorBase)))
 	{
-		scr->xinerama_screens = safemalloc (sizeof (XRectangle) * scr->xinerama_screens_num);
-		for (i = 0; i < scr->xinerama_screens_num; ++i)
+		register int  i;
+		XineramaScreenInfo *s;
+
+		if ((s = XineramaQueryScreens (dpy, &(scr->xinerama_screens_num))) != NULL)
 		{
-			scr->xinerama_screens[i].x = s[i].x_org;
-			scr->xinerama_screens[i].y = s[i].y_org;
-			scr->xinerama_screens[i].width = s[i].width;
-			scr->xinerama_screens[i].height = s[i].height;
+			scr->xinerama_screens = safemalloc (sizeof (XRectangle) * scr->xinerama_screens_num);
+	    	asxml_var_insert("xroot.xinerama_screens_num", scr->xinerama_screens_num);
+			static char buf[256] ; 
+			for (i = 0; i < scr->xinerama_screens_num; ++i)
+			{
+				char *append_point = &buf[0]; 
+				sprintf( append_point, "xroot.xinerama_screens[%d].", i );
+				append_point += 23 ; 
+				while( *append_point ) ++append_point ; 
+
+				append_point[0] = 'x' ;append_point[1] = '\0' ;
+		    	asxml_var_insert(&buf[0], s[i].x_org);
+				scr->xinerama_screens[i].x = s[i].x_org;
+
+				append_point[0] = 'y' ;
+		    	asxml_var_insert(&buf[0], s[i].y_org);
+				scr->xinerama_screens[i].y = s[i].y_org;
+
+				strcpy( append_point, "width" );
+		    	asxml_var_insert(&buf[0], s[i].width);
+				scr->xinerama_screens[i].width = s[i].width;
+
+				strcpy( append_point, "height" );
+		    	asxml_var_insert(&buf[0], s[i].height);
+				scr->xinerama_screens[i].height = s[i].height;
+			}
+			XFree (s);
 		}
-		XFree (s);
-	}
-#else
-	scr->xinerama_screens = NULL;
-	scr->xinerama_screens_num = 0;
+	}else
 #endif
+	{
+		scr->xinerama_screens = NULL;
+		scr->xinerama_screens_num = 0;
+    	asxml_var_insert("xroot.xinerama_screens_num", 0);
+	}
 }
 
 Bool
@@ -323,14 +346,14 @@ ConnectXDisplay (Display *display, ScreenInfo * scr, Bool as_manager)
 	button_h = (display_dpcmy < 50)?48:64;
 	mini_w = (display_dpcmx <= 44)?max(display_dpcmx/2,12):display_dpcmx-20;
 	mini_h = (display_dpcmy <= 44)?max(display_dpcmy/2,12):display_dpcmy-20;
-	asxml_var_insert("icon.button.width", button_w);
-	asxml_var_insert("icon.button.height", button_h);
-	asxml_var_insert("icon.width", (button_w*3)/4);
-	asxml_var_insert("icon.height", (button_h*3)/4);
-	asxml_var_insert("minipixmap.width", mini_w);
-	asxml_var_insert("minipixmap.height", mini_h);
-	asxml_var_insert("title.font.size", (mini_h*7)/12);
-	asxml_var_insert("menu.font.size", (mini_h*7)/12+1);
+	asxml_var_insert(ASXMLVAR_IconButtonWidth, button_w);
+	asxml_var_insert(ASXMLVAR_IconButtonHeight, button_h);
+	asxml_var_insert(ASXMLVAR_IconWidth, (button_w*3)/4);
+	asxml_var_insert(ASXMLVAR_IconHeight, (button_h*3)/4);
+	asxml_var_insert(ASXMLVAR_MinipixmapWidth, mini_w);
+	asxml_var_insert(ASXMLVAR_MinipixmapHeight, mini_h);
+	asxml_var_insert(ASXMLVAR_TitleFontSize, (mini_h*7)/12);
+	asxml_var_insert(ASXMLVAR_MenuFontSize, (mini_h*7)/12+1);
 	asxml_var_insert("font.size", ((mini_h*7)/12));
 
 	scr->CurrentDesk = -1;
@@ -355,10 +378,8 @@ ConnectXDisplay (Display *display, ScreenInfo * scr, Bool as_manager)
 
     setup_modifiers ();
 
-#ifdef HAVE_XINERAMA
-	if (XineramaQueryExtension (dpy, &(scr->XineEventBase), &(scr->XineErrorBase)))
-		get_Xinerama_rectangles (scr);
-#endif
+	get_Xinerama_rectangles (scr);
+
 #ifdef SHAPE
 	XShapeQueryExtension (dpy, &(scr->ShapeEventBase), &(scr->ShapeErrorBase));
 #endif /* SHAPE */
@@ -446,7 +467,9 @@ init_screen_gcs(ScreenInfo *scr)
 		scr = ASDefaultScr;
 	
 	gcv.graphics_exposures = False;
-    scr->DrawGC = create_visual_gc( scr->asv, scr->Root, gcm, &gcv );
+
+    scr->RootGC = XCreateGC( dpy, scr->Root, gcm, &gcv );
+	scr->DrawGC = create_visual_gc( scr->asv, scr->Root, gcm, &gcv );
 }
 
 void
@@ -460,6 +483,11 @@ destroy_screen_gcs(ScreenInfo *scr)
 		XFreeGC(dpy, scr->DrawGC);
 		scr->DrawGC = NULL ;
 	}
+	if( scr->RootGC )
+	{	
+		XFreeGC(dpy, scr->RootGC);
+		scr->RootGC = NULL ;
+	}
 }		
 
 /* Setting up global variables  nonlock_mods, and lock_mods, defined in asapp.c : */
