@@ -195,7 +195,9 @@ CheckConfigSanity()
     sprintf( buf, "*%sTile", get_application_name() );
     LOCAL_DEBUG_OUT("Attempting to use style \"%s\"", buf);
     Scr.Look.MSWindow[BACK_UNFOCUSED] = mystyle_find_or_default( buf );
-    LOCAL_DEBUG_OUT("Will use style \"%s\"", Scr.Look.MSWindow[BACK_UNFOCUSED]->name);
+    LOCAL_DEBUG_OUT("Will use style \"%s\", overlay = %p.", 
+						Scr.Look.MSWindow[BACK_UNFOCUSED]->name,
+						Scr.Look.MSWindow[BACK_UNFOCUSED]->overlay );
 }
 
 
@@ -352,7 +354,7 @@ make_ident_window( int width, int height)
 	}
     attr.event_mask = StructureNotifyMask ;
     w = create_visual_window( Scr.asv, Scr.Root, x, y, width, height, 0, InputOutput, CWEventMask, &attr);
-    set_client_names( w, MyName, MyName, CLASS_IDENT, MyName );
+    set_client_names( w, MyName, MyName, AS_MODULE_CLASS, CLASS_IDENT );
 
     Scr.RootClipArea.x = x;
     Scr.RootClipArea.y = y;
@@ -454,6 +456,9 @@ struct
 	{STYLE_FOCUS_ON_MAP, 		"FocusOnMap", 			"NoFocusOnMap"},
 	{STYLE_LONG_LIVING,			"LongLiving",			"ShortLiving"},
 	{STYLE_IGNORE_CONFIG,  		"IgnoreConfig",			"HonorConfig"},
+	{STYLE_IGNORE_RESTACK, 		"IgnoreRestack",		"HonorRestack"},
+	{STYLE_CURRENT_VIEWPORT,    "UseCurrentViewport",   "UseAnyViewport"},
+
 	{0,NULL, NULL}};
 
 const char *Gravity2text[] =
@@ -559,6 +564,10 @@ fill_window_data()
 	SHOW_FLAG(wd->client_icon_flags,ClientIconPosition);
 	SHOW_FLAG(wd->flags,FocusOnMap);          
 	SHOW_FLAG(wd->flags,ShortLived);
+	SHOW_FLAG(wd->flags,Module);
+	SHOW_FLAG(wd->flags,IgnoreConfigRequest);
+	SHOW_FLAG(wd->flags,IgnoreRestackRequest);
+	SHOW_FLAG(wd->flags,WMDockApp);
 	
 	add_property("Current hints flags:", buf, AS_Text_ASCII, True);
 	
@@ -571,9 +580,6 @@ fill_window_data()
 	SHOW_FLAG(wd->flags,SizeInc);
 	SHOW_FLAG(wd->flags,BaseSize);
 	SHOW_FLAG(wd->flags,Gravity);
-	SHOW_FLAG(wd->flags,Module);
-	SHOW_FLAG(wd->flags,IgnoreConfigRequest);
-	SHOW_FLAG(wd->flags,WMDockApp);
 	
 	add_property("Specified hint values:", buf, AS_Text_ASCII, True);
 
@@ -786,10 +792,15 @@ fill_window_data()
 		add_property("Matched Styles:", "", AS_Text_ASCII, True);
 		for( i = 0 ; Database->match_list[i] >= 0 ; ++i )
 		{
-			ASDatabaseRecord *dr = &(Database->styles_table[Database->match_list[i]]) ;
+			ASDatabaseRecord *dr = 	get_asdb_record (Database, Database->match_list[i]);
 			int f ;
 			Bool first = True ;
-			sprintf( buf, "\"%s\" \t", dr->regexp->raw );
+			if( is_default_asdb_record(Database, dr) ) 
+				sprintf(buf, "\"*\" \t");
+			else if( dr->regexp ) 
+				sprintf( buf, "\"%s\" \t", dr->regexp->raw );
+			else
+				sprintf( buf, "\"(none)\" \t" );
 
 			for( f = 0 ; DatabaseFlagsList[f].name_set != NULL ; ++f ) 
 			{	
@@ -977,6 +988,11 @@ add_property( const char *name, const char *value, unsigned long value_encoding,
 	if( IdentState.used_props < MAX_PROPERTIES )
 	{
 		int tmp ;
+		
+		if( value && strlen(value) > 0 ) 
+			printf( (strlen(name)>15)?"%s\t%s;\n":"%s\t\t%s;\n", name, value );
+		else
+			printf( "%s\n", name );
 		IdentState.props[IdentState.used_props].property_name = mystrdup(name);
 		IdentState.props[IdentState.used_props].value = mystrdup(value);
 		IdentState.props[IdentState.used_props].label_bar = create_astbar();
