@@ -185,6 +185,7 @@ void update_tabs_desktop();
 void update_tabs_state();
 void register_unswallowed_client( Window client );
 Bool check_unswallowed_client( Window client ); 
+void delete_tab( int index );
 
 /* above function may also return : */
 #define BANNER_TAB_INDEX -1		   
@@ -291,7 +292,6 @@ for( i = 1 ; i< argc ; ++i)
 	signal (SIGTERM, DeadPipe);
     signal (SIGKILL, DeadPipe);
     
-    balloon_init (False);
     Config = CreateWinTabsConfig ();
 
     LoadBaseConfig ( GetBaseOptions);
@@ -366,8 +366,24 @@ LOCAL_DEBUG_OUT( "DeadPipe%s", "" );
     
 		ASSync(False );
     }	  
-	if( WinTabsState.main_canvas )
-    	destroy_ascanvas( &WinTabsState.main_canvas );
+	
+	while( PVECTOR_USED( WinTabsState.tabs ) ) 
+		delete_tab(0);		
+	destroy_string( &(WinTabsState.banner.name) );
+   	destroy_astbar( &(WinTabsState.banner.bar) );
+   	destroy_ascanvas( &(WinTabsState.banner.client_canvas) );
+   	destroy_ascanvas( &(WinTabsState.banner.frame_canvas) );
+
+	destroy_asvector( &WinTabsState.tabs );
+   	destroy_ascanvas( &WinTabsState.tabs_canvas );
+   	destroy_ascanvas( &WinTabsState.main_canvas );
+
+	destroy_astbar_props( &(WinTabsState.tbar_props) );
+	free_button_resources( &WinTabsState.close_button );
+	free_button_resources( &WinTabsState.unswallow_button );
+	free_button_resources( &WinTabsState.menu_button );
+
+
     if( WinTabsState.main_window )
         XDestroyWindow( dpy, WinTabsState.main_window );
 	ASSync(False );
@@ -393,6 +409,13 @@ void
 retrieve_wintabs_astbar_props()
 {
 	destroy_astbar_props( &(WinTabsState.tbar_props) );
+
+	free_button_resources( &WinTabsState.close_button );
+	free_button_resources( &WinTabsState.unswallow_button );
+	free_button_resources( &WinTabsState.menu_button );
+	memset(&WinTabsState.close_button, 0x00, sizeof(MyButton));
+	memset(&WinTabsState.unswallow_button, 0x00, sizeof(MyButton));
+	memset(&WinTabsState.menu_button, 0x00, sizeof(MyButton));
 	
 	WinTabsState.tbar_props = get_astbar_props(Scr.wmprops );
 	button_from_astbar_props( WinTabsState.tbar_props, &WinTabsState.close_button, 		C_CloseButton, 		_AS_BUTTON_CLOSE, _AS_BUTTON_CLOSE_PRESSED );
@@ -417,13 +440,13 @@ CheckConfigSanity(const char *pattern_override, const char *exclude_pattern_over
 	if( MyArgs.geometry.flags != 0 ) 
 		Config->geometry = MyArgs.geometry ;
 	if( pattern_override ) 
-		set_string_value(&(Config->pattern), mystrdup(pattern_override), NULL, 0);
+		set_string(&(Config->pattern), mystrdup(pattern_override));
 	if( exclude_pattern_override ) 
-		set_string_value(&(Config->exclude_pattern), mystrdup(exclude_pattern_override), NULL, 0);
+		set_string(&(Config->exclude_pattern), mystrdup(exclude_pattern_override) );
    	if( title_override ) 
-		set_string_value(&(Config->title), mystrdup(title_override), NULL, 0 );
+		set_string(&(Config->title), mystrdup(title_override) );
    	if( icon_title_override ) 
-		set_string_value(&(Config->icon_title), mystrdup(icon_title_override), NULL, 0 );
+		set_string(&(Config->icon_title), mystrdup(icon_title_override) );
 
 	if( Config->icon_title == NULL && Config->title != NULL ) 
 	{
@@ -479,7 +502,8 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
     if( get_flags(Config->geometry.flags, YNegative) )
         Config->anchor_y += Scr.MyDisplayHeight ;
 
-    retrieve_wintabs_astbar_props();
+	if( WinTabsState.tbar_props == NULL ) 
+	    retrieve_wintabs_astbar_props();
 	mystyle_get_property (Scr.wmprops);
 
     Scr.Look.MSWindow[BACK_UNFOCUSED] = mystyle_find( Config->unfocused_style );
@@ -488,11 +512,19 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
 
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
-        static char *default_window_style_name[BACK_STYLES] ={"focused_window_style","unfocused_window_style","sticky_window_style", NULL};
+        static char *default_window_style_name[BACK_STYLES] ={"focused_window_style","unfocused_window_style","sticky_window_style", NULL, NULL};
         if( Scr.Look.MSWindow[i] == NULL )
             Scr.Look.MSWindow[i] = mystyle_find( default_window_style_name[i] );
-        if( Scr.Look.MSWindow[i] == NULL )
+        if( Scr.Look.MSWindow[i] == NULL && i != BACK_URGENT )
             Scr.Look.MSWindow[i] = mystyle_find_or_default( default_style );
+		if( Scr.Look.MSWindow[i] )
+		{
+			if( Scr.Look.MSWindow[i]->texture_type == TEXTURE_SHAPED_PIXMAP )
+				Scr.Look.MSWindow[i]->texture_type = TEXTURE_PIXMAP ;
+			else if( Scr.Look.MSWindow[i]->texture_type == TEXTURE_SHAPED_SCALED_PIXMAP )
+				Scr.Look.MSWindow[i]->texture_type = TEXTURE_SCALED_PIXMAP ;
+			
+		}
     }
     free( default_style );
 
@@ -501,20 +533,7 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
     Print_balloonConfig ( Config->balloon_conf );
 #endif
 	
-	balloon_config2look( &(Scr.Look), Config->balloon_conf );
-	if( Config->balloon_conf == NULL ) 
-	{
-/*		
-		Scr.Look.balloon_look->show = True ;
-        Scr.Look.balloon_look->xoffset = 5 ;
-        Scr.Look.balloon_look->yoffset = 5 ;
-		Scr.Look.balloon_look->style = mystyle_list_find_or_default (NULL, NULL);
-		Scr.Look.balloon_look->close_delay = 2000 ;
- */
-	}	 
-		
-    LOCAL_DEBUG_OUT( "balloon mystyle = %p (\"%s\")", Scr.Look.balloon_look->style,
-                    Scr.Look.balloon_look->style?Scr.Look.balloon_look->style->name:"none" );
+	balloon_config2look( &(Scr.Look), NULL, Config->balloon_conf, "*WinTabsBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
 
 }
@@ -547,7 +566,7 @@ GetOptions (const char *filename)
 
     if( config->pattern )
     {    
-        set_string_value( &(Config->pattern), mystrdup(config->pattern), NULL, 0 );
+        set_string( &(Config->pattern), mystrdup(config->pattern) );
         Config->pattern_type = config->pattern_type ; 
     }
     if( get_flags(config->set_flags, WINTABS_MaxRows) )
@@ -560,11 +579,11 @@ GetOptions (const char *filename)
         Config->max_tab_width = config->max_tab_width;
 
     if( config->unfocused_style )
-        set_string_value( &(Config->unfocused_style), mystrdup(config->unfocused_style), NULL, 0 );
+        set_string( &(Config->unfocused_style), mystrdup(config->unfocused_style) );
     if( config->focused_style )
-        set_string_value( &(Config->focused_style), mystrdup(config->focused_style), NULL, 0 );
+        set_string( &(Config->focused_style), mystrdup(config->focused_style) );
     if( config->sticky_style )
-        set_string_value( &(Config->sticky_style), mystrdup(config->sticky_style), NULL, 0 );
+        set_string( &(Config->sticky_style), mystrdup(config->sticky_style) );
 
     if( get_flags(config->set_flags, WINTABS_Align) )
         Config->name_aligment = config->name_aligment;
@@ -698,7 +717,13 @@ DispatchEvent (ASEvent * event)
 		int pointer_root_x = event->x.xkey.x_root;
 		int pointer_root_y = event->x.xkey.y_root;
 		static int last_pointer_root_x = -1, last_pointer_root_y = -1; 
-        
+	
+		if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+		{
+			withdraw_balloon(NULL);
+			return;
+		}
+
 		pointer_tab  = find_tab_by_position( pointer_root_x, pointer_root_y );
         LOCAL_DEBUG_OUT( "pointer at %d,%d - pointer_tab = %d", event->x.xmotion.x_root, event->x.xmotion.y_root, pointer_tab );
 		if( pointer_tab == BANNER_TAB_INDEX ) 
@@ -852,7 +877,6 @@ DispatchEvent (ASEvent * event)
 			}
 	        break;
 	    case PropertyNotify:
-			LOCAL_DEBUG_OUT( "property %s(%lX), _XROOTPMAP_ID = %lX, event->w = %lX, root = %lX", XGetAtomName(dpy, event->x.xproperty.atom), event->x.xproperty.atom, _XROOTPMAP_ID, event->w, Scr.Root );
 			if( event->w == Scr.Root || event->w == Scr.wmprops->selection_window ) 
 			{
 				if( event->w == Scr.Root && event->x.xproperty.atom == _XA_WIN_SUPPORTING_WM_CHECK )
@@ -1046,7 +1070,9 @@ make_wintabs_window()
     unsigned int width = max(Config->geometry.width,1);
     unsigned int height = max(Config->geometry.height,1);
     XSetWindowAttributes attributes;
+	char *iconic_name ; 
 
+	memset( &extwm_hints, 0x00, sizeof(extwm_hints));
     attributes.background_pixmap = ParentRelative;
     switch( Config->gravity )
 	{
@@ -1070,17 +1096,30 @@ make_wintabs_window()
 	}
     LOCAL_DEBUG_OUT( "creating main window with geometry %dx%d%+d%+d", width, height, x, y );
     w = create_visual_window( Scr.asv, Scr.Root, x, y, 1, 1, 0, InputOutput, CWBackPixmap, &attributes);
-    set_client_names( w, Config->title?Config->title:"WinTabs", 
-						 Config->icon_title?Config->icon_title:"WinTabs - iconic", 
-					  	 CLASS_WINTABS, MyName );
+    LOCAL_DEBUG_OUT( "main window created with Id %lX", w);
+
+	iconic_name = Config->icon_title ; 
+	if( iconic_name == NULL ) 
+	{
+		iconic_name = safemalloc( strlen(MyName)+32 );
+		sprintf( iconic_name, "%s iconic", MyName );
+	}	
+    set_client_names( w, Config->title?Config->title:MyName, 
+						 iconic_name, 
+					  	 AS_MODULE_CLASS, CLASS_WINTABS );
+	if( iconic_name != Config->icon_title ) 
+		free( iconic_name );
 
     shints.flags = USPosition|USSize|PWinGravity;
     shints.win_gravity = Config->gravity ;
 
 	extwm_hints.pid = getpid();
-    extwm_hints.flags = EXTWM_PID|EXTWM_TypeASModule|EXTWM_TypeNormal ;
+    extwm_hints.flags = EXTWM_PID|EXTWM_TypeSet ;
+	extwm_hints.type_flags = EXTWM_TypeASModule|EXTWM_TypeNormal ;
+	
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
+	set_client_cmd (w);
 
     /* we will need to wait for PropertyNotify event indicating transition
 	   into Withdrawn state, so selecting event mask: */
@@ -1608,7 +1647,7 @@ do_swallow_window( ASWindowData *wd )
 	set_client_desktop (wd->client, WinTabsState.my_desktop );	 
 
 #if 0   /* TODO : implement support for icons : */
-    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) &&
+    if( get_flags( wd->client_icon_flags, AS_ClientIcon ) && !get_flags( wd->client_icon_flags, AS_ClientIconPixmap) &&
 		wd->icon != None )
     {
         ASCanvas *ic = create_ascanvas_container( wd->icon );
@@ -1875,6 +1914,7 @@ Bool handle_tab_name_change( Window client)
 		ASSupportedHints *list = create_hints_list ();
 		
 		enable_hints_support (list, HINTS_ICCCM);
+		enable_hints_support (list, HINTS_KDE);
 		enable_hints_support (list, HINTS_ExtendedWM);
 		
 		memset( &raw, 0x00, sizeof(ASRawHints));
