@@ -118,6 +118,8 @@ typedef struct ASWindow
 #define ASWIN_CLEAR_FLAGS(t,f) 	clear_flags((t)->status->flags,(f))
 #define ASWIN_FUNC_MASK(t)  ((t)->hints->function_mask)
 
+#define ASWIN_FOCUSABLE(asw)  (ASWIN_HFLAGS(asw, AS_AcceptsFocus) || get_flags((asw)->hints->protocols, AS_DoesWmTakeFocus))
+
 #define IsBtnDisabled(t,b)  get_flags((t)->hints->disabled_buttons,(0x01<<(b)))
 #define IsBtnEnabled(t,b)   (!get_flags((t)->hints->disabled_buttons,(0x01<<(b))))
 #define DisableBtn(t,b)  	clear_flags((t)->hints->disabled_buttons,(0x01<<(b)))
@@ -243,6 +245,9 @@ typedef struct ASWindowList
      *          some other window. To do that we substitute focused pointer to new window
      *          right before UngrabEm. In this case focused != ( ungrabbed == hilited )
      */
+
+	ASVector	*stacking_order ; 		/* array of pointers to ASWindow structures */
+	
 }ASWindowList;
 
 /* Mirror Note :
@@ -349,6 +354,10 @@ extern ASFlagType    AfterStepState;              /* see ASS_ flags above */
 extern struct ASDatabase *Database;
 extern ASHashTable       *ComplexFunctions;
 
+extern ASBalloonState *MenuBalloons ; 
+extern ASBalloonState *TitlebarBalloons ; 
+
+
 /* global variables for Look values : */
 extern unsigned long XORvalue;
 extern int           RubberBand;
@@ -404,6 +413,7 @@ ASWindow *bookmark2ASWindow( const char *bookmark );
 ASWindow *pattern2ASWindow( const char *pattern );
 ASWindow *complex_pattern2ASWindow( char *pattern );
 ASLayer *get_aslayer( int layer, ASWindowList *list );
+ASWindow *find_group_lead_aswindow( Window id );
 void tie_aswindow( ASWindow *t );
 void untie_aswindow( ASWindow *t );
 void add_aswindow_to_layer( ASWindow *asw, int layer );
@@ -411,7 +421,15 @@ void remove_aswindow_from_layer( ASWindow *asw, int layer );
 Bool enlist_aswindow( ASWindow *t );
 void delist_aswindow( ASWindow *t );
 void save_aswindow_list( ASWindowList *list, char *file );
-void restack_window_list( int desk, Bool send_msg_only );
+ASWindow* find_topmost_client( int desk, int root_x, int root_y );
+void free_scratch_layers_vector();
+void free_scratch_ids_vector();
+void update_stacking_order();
+void restack_window_list( int desk);
+void send_stacking_order( int desk );
+void apply_stacking_order( int desk );
+
+
 Bool is_window_obscured (ASWindow * above, ASWindow * below);
 void restack_window( ASWindow *t, Window sibling_window, int stack_mode );
 
@@ -434,7 +452,10 @@ MenuData *make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order,
 
 void hide_focus();
 Bool focus_window( ASWindow *asw, Window w );
-Bool focus_aswindow( ASWindow *asw );
+void autoraise_window( ASWindow *asw );
+#define FOCUS_ASW_CAN_AUTORAISE		False
+#define FOCUS_ASW_DONT_AUTORAISE	True
+Bool focus_aswindow( ASWindow *asw, Bool suppress_autoraise );
 Bool focus_active_window();
 void focus_next_aswindow( ASWindow *asw );     /* should be called when window is unmapped or destroyed */
 void focus_prev_aswindow( ASWindow *asw );     /* should be called when window is unmapped or destroyed */
@@ -457,7 +478,7 @@ void on_icon_changed( ASWindow *asw );
 void on_window_title_changed( ASWindow *asw, Bool update_display );
 void on_window_hints_changed( ASWindow *asw );
 void on_window_opacity_changed( ASWindow *asw );
-void on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured );
+void on_window_status_changed( ASWindow *asw, Bool reconfigured );
 void on_window_hilite_changed( ASWindow *asw, Bool focused );
 void on_window_pressure_changed( ASWindow *asw, int pressed_context );
 
@@ -691,6 +712,7 @@ void release_all_old_background( Bool forget );
 
 /******************************* placement.c *******************************/
 ASGrid* make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWindow *target );
+void setup_aswindow_moveresize(ASWindow *asw,  struct ASMoveResizeData *mvrdata );
 Bool place_aswindow( ASWindow *asw );
 void apply_aswindow_move(struct ASMoveResizeData *data);
 void apply_aswindow_moveresize(struct ASMoveResizeData *data);
