@@ -55,6 +55,163 @@
 #include "../../libAfterConf/afterconf.h"
 
 
+typedef struct AfterStepConfig
+{
+	ASModuleConfig asmodule_config;
+	ASFlagType	flags ;
+	ASFlagType	set_flags ;
+
+}AfterStepConfig;
+
+#define AS_AFTERSTEP_CONFIG(p) AS_MODULE_CONFIG_TYPED(p,CONFIG_AfterStep_ID,AfterStepConfig)
+
+
+static void
+InitAfterStepConfig (ASModuleConfig *asm_config, Bool free_resources)
+{
+	AfterStepConfig *config = AS_AFTERSTEP_CONFIG(asm_config);
+	if( config ) 
+	{
+		/* TODO */
+		if( free_resources ) 
+		{
+		}
+	}
+}
+
+void
+AfterStep_fs2config( ASModuleConfig *asmodule_config, FreeStorageElem *Storage )
+{
+	FreeStorageElem *pCurr;
+	ConfigItem    item;
+	AfterStepConfig *config = AS_AFTERSTEP_CONFIG(asmodule_config) ;
+	
+	if( config == NULL ) 
+		return ; 
+	
+	item.memory = NULL;
+    for (pCurr = Storage; pCurr; pCurr = pCurr->next)
+	{
+		if (pCurr->term == NULL)
+			continue;
+
+		if (pCurr->term->type == TT_FLAG)
+        {
+        }else
+		{
+			if (!ReadConfigItem (&item, pCurr))
+				continue;
+
+			switch (pCurr->term->id)
+			{
+		        default:
+        		    item.ok_to_free = 1;
+			}
+		}
+	}
+	
+	ReadConfigItem (&item, NULL);
+}
+
+void
+MergeAfterStepOptions ( ASModuleConfig *asm_to, ASModuleConfig *asm_from)
+{
+    int i ;
+    START_TIME(option_time);
+
+	AfterStepConfig *to = AS_AFTERSTEP_CONFIG(asm_to);
+	AfterStepConfig *from = AS_AFTERSTEP_CONFIG(asm_from);
+	if( to && from )
+	{
+
+    	/* Need to merge new config with what we have already :*/
+    	/* now lets check the config sanity : */
+    	/* mixing set and default flags : */
+    	ASCF_MERGE_FLAGS(to,from);
+
+	}
+    SHOW_TIME("to parsing",option_time);
+}
+
+
+flag_options_xref AfterStepConfigFlags[] = {
+/*	ASCF_DEFINE_MODULE_FLAG_XREF(WINLIST,FillRowsFirst,WinListConfig), */
+    {0, 0, 0}
+};
+
+
+int AfterStepBalloons[] = { TITLE_BALLOON_ID_START, MENU_BALLOON_ID_START, 0 };
+
+static ASModuleConfigClass _afterstep_config_class = 
+{	CONFIG_AfterStep_ID,
+ 	ASMC_HandlePublicLookOptions|
+	ASMC_HandlePublicFeelOptions
+	/* |ASMC_HandleLookMyStyles */,
+	sizeof(AfterStepConfig),
+ 	"afterstep",
+ 	InitAfterStepConfig,
+	AfterStep_fs2config,
+	MergeAfterStepOptions,
+	&AfterStepSyntax,
+	&LookSyntax,
+	&FeelSyntax,
+	AfterStepConfigFlags,
+	offsetof(AfterStepConfig,set_flags),
+	
+	AfterStepBalloons
+ };
+ 
+ASModuleConfigClass *AfterStepConfigClass = &_afterstep_config_class;
+
+
+void 
+ReloadConfig(ASFlagType what)
+{
+	ASBalloonLook ballon_look ; 
+
+	ASModuleConfig *config = parse_asmodule_config_all( AfterStepConfigClass );
+
+	/* apply it  */
+	Print_balloonConfig (config->balloon_configs[0] );
+    balloon_config2look( &Scr.Look, &ballon_look, config->balloon_configs[0], "TitleButtonBalloon" );
+    set_balloon_state_look( TitlebarBalloons, &ballon_look );
+	Print_balloonConfig (config->balloon_configs[1] );
+    balloon_config2look( &Scr.Look, &ballon_look, config->balloon_configs[1], "MenuBalloon" );
+	set_balloon_state_look( MenuBalloons,  &ballon_look );
+	
+	
+	destroy_ASModule_config( config );
+}
+
+#ifdef AFTERSTEP_CONFIG_TEST
+
+ASBalloonState *MenuBalloons = NULL ; 
+ASBalloonState *TitlebarBalloons = NULL ; 
+
+int main(int argc, char **argv )
+{
+
+    ASImageManager  *old_image_manager = NULL ;
+    ASFontManager   *old_font_manager  = NULL ;
+    InitMyApp( CLASS_AFTERSTEP, argc, argv, NULL, NULL, 0);
+	LinkAfterStepConfig();
+    if( ConnectX( ASDefaultScr, 0 ) < 0  )
+	{
+		show_error( "Hostile X server encountered - unable to proceed :-(");
+		return 1;/* failed to accure window management selection - other wm is running */
+	}
+	InitSession();
+	MenuBalloons = create_balloon_state(); 
+	TitlebarBalloons = create_balloon_state(); 
+	ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True, True );
+	LoadColorScheme();
+	ReloadConfig(0xFFFFFFFF);
+
+}
+
+#else
+
+
 /* old look auxilary variables : */
 static MyFont StdFont    = {NULL};         /* font structure */
 static MyFont WindowFont = {NULL};      /* font structure for window titles */
@@ -90,7 +247,8 @@ static MyStyleDefinition *MyStyleList = NULL ;
 static MyFrameDefinition *MyFrameList = NULL ;
 static MyFrameDefinition *LegacyFrameDef = NULL ;
 
-static balloonConfig BalloonConfig = {0, 0, 0, 0, 0, 0, NULL };
+static balloonConfig BalloonConfig = {0, 0, 0, 0, 0, 0, 0, NULL, 0, 0 };
+static balloonConfig MenuBalloonConfig = {0, 0, 0, 0, 0, 0, 0, NULL, 0, 0 };
 
 static char         *MSWindowName[BACK_STYLES] = {NULL};
 static char         *MSMenuName[MENU_BACK_STYLES] = {NULL };
@@ -270,6 +428,7 @@ struct config main_config[] = {
     {"TitlebarNoPush"					, SetLookFlag, (char **)TitlebarNoPush, NULL},
     {"TextureMenuItemsIndividually"		, SetLookFlag, (char **)TxtrMenuItmInd,NULL},
     {"MenuMiniPixmaps"					, SetLookFlag, (char **)MenuMiniPixmaps, NULL},
+	{"MenuShowUnavailable"				, SetLookFlag, (char **)MenuShowUnavailable, NULL },
 	{"TitleTextAlign"					, SetInts, (char **)&TmpLook.TitleTextAlign, &dummy},
     {"TitleButtonSpacingLeft"			, SetInts, (char **)&TmpLook.TitleButtonSpacing[0], &dummy},
     {"TitleButtonSpacingRight"			, SetInts, (char **)&TmpLook.TitleButtonSpacing[1], &dummy},
@@ -302,6 +461,15 @@ struct config main_config[] = {
     {"MenuItemCompositionMethod"		, SetInts, (char **)&TmpLook.menu_icm, &dummy},
     {"MenuHiliteCompositionMethod"		, SetInts, (char **)&TmpLook.menu_hcm, &dummy},
     {"MenuStippleCompositionMethod"		, SetInts, (char **)&TmpLook.menu_scm, &dummy},
+    {"MenuBalloonBorderHilite"			, bevel_parse, (char**)"afterstep", (int*)&(MenuBalloonConfig.BorderHilite)},
+    {"MenuBalloonXOffset"				, SetInts, (char**)&(MenuBalloonConfig.XOffset), NULL},
+    {"MenuBalloonYOffset"				, SetInts, (char**)&(MenuBalloonConfig.YOffset), NULL},
+    {"MenuBalloonDelay"					, SetInts, (char**)&(MenuBalloonConfig.Delay), NULL},
+    {"MenuBalloonCloseDelay"			, SetInts, (char**)&(MenuBalloonConfig.CloseDelay), NULL},
+    {"MenuBalloonStyle"					, assign_string,   &(MenuBalloonConfig.Style), NULL},
+    {"MenuBalloonTextPaddingX"			, SetInts, (char**)&(MenuBalloonConfig.TextPaddingX), NULL},
+    {"MenuBalloonTextPaddingY"			, SetInts, (char**)&(MenuBalloonConfig.TextPaddingY), NULL},
+    {"MenuBalloons"						, SetFlag2, (char**)BALLOON_USED, (int*)&(MenuBalloonConfig.set_flags)},
     {"ShadeAnimationSteps"				, SetInts, (char **)&TmpFeel.ShadeAnimationSteps, (int *)&dummy},
     {"TitleButtonBalloonBorderHilite"	, bevel_parse, (char**)"afterstep", (int*)&(BalloonConfig.BorderHilite)},
     {"TitleButtonBalloonXOffset"		, SetInts, (char**)&(BalloonConfig.XOffset), NULL},
@@ -314,7 +482,7 @@ struct config main_config[] = {
     {"TitleButtonBalloons"				, SetFlag2, (char**)BALLOON_USED, (int*)&(BalloonConfig.set_flags)},
     {"TitleButton"						, SetTitleButton, (char **)1, (int *)0},
     {"KillBackgroundThreshold"			, SetInts, (char**)&(TmpLook.KillBackgroundThreshold), NULL },
-	{"DontAnimateBackground"			, SetFlag, (char**)DontAnimateBackground, &dummy},
+	{"DontAnimateBackground"			, SetFlag2, (char**)DontAnimateBackground, &dummy},
 	{"CoverAnimationSteps"				, SetInts, (char**)&(TmpFeel.desk_cover_animation_steps), NULL },
 	{"CoverAnimationType"				, SetInts, (char**)&(TmpFeel.desk_cover_animation_type), NULL },
 	{"AnimateDeskChange"				, SetFlag, (char**)AnimateDeskChange, &dummy },
@@ -659,6 +827,7 @@ void
 ApplyFeel( ASFeel *feel )
 {
     check_screen_panframes(ASDefaultScr);
+	
 }
 
 /*
@@ -749,11 +918,6 @@ merge_look( MyLook *to, MyLook *from )
     to->minipixmap_width  = (from->minipixmap_width == 0)? 24 : from->minipixmap_width ; 
 	to->minipixmap_height = (from->minipixmap_height == 0)?to->minipixmap_width:from->minipixmap_height; 
 	
-	asxml_var_insert("icon.button.width", to->ButtonWidth);
-	asxml_var_insert("icon.button.height", to->ButtonHeight);
-
-	asxml_var_insert("minipixmap.width", to->minipixmap_width);
-	asxml_var_insert("minipixmap.height", to->minipixmap_height);
 
 	to->DefaultFrameName = from->DefaultFrameName ; 
 
@@ -970,11 +1134,13 @@ fix_menu_pin_on( MyLook *look )
         if( MenuPinOnButton < 0 || MenuPinOnButton >= TITLE_BUTTONS )
         {
             register int i = TITLE_BUTTONS ;
+
             while ( --i >= 0 )
             {
                 if( look->buttons[i].unpressed.image == NULL && look->buttons[i].pressed.image == NULL )
                     break;
             }
+			
             if( i >= 0 )
             {
                 if( GetIconFromFile (MenuPinOn, &(Scr.Look.buttons[i].unpressed), 0) )
@@ -1017,18 +1183,12 @@ fix_menu_pin_on( MyLook *look )
         }
         show_warning( "MenuPinOn setting is depreciated - instead add a Title button and bind PinMenu function to it." );
     }
-#ifdef LOCAL_DEBUG
-    LOCAL_DEBUG_OUT( "syncing %s","");
-    ASSync(False);
-#endif
-
 }
 
 void
 FixLook( MyLook *look )
 {
     ASFlagType default_title_align = ALIGN_LEFT ;
-	int tbar_font_size = 0 ;
 	int menu_font_size = 0 ;
     int i ;
 #ifdef LOCAL_DEBUG
@@ -1070,19 +1230,14 @@ FixLook( MyLook *look )
     ASSync(False);
 #endif
 
-	for( i = 0 ; i < BACK_STYLES ; ++i ) 
-		if( look->MSWindow[i] )
-			if( look->MSWindow[i]->font.as_font )
-				if( look->MSWindow[i]->font.as_font->max_height > tbar_font_size ) 
-					tbar_font_size = look->MSWindow[i]->font.as_font->max_height ;
-	asxml_var_insert("title.font.size", tbar_font_size);
+	mylook_set_font_size_var (look);
 	
 	for( i = 0 ; i < MENU_BACK_STYLES ; ++i ) 
 		if( look->MSMenu[i] )
 			if( look->MSMenu[i]->font.as_font )
 				if( look->MSMenu[i]->font.as_font->max_height > menu_font_size ) 
 					menu_font_size = look->MSMenu[i]->font.as_font->max_height ;
-	asxml_var_insert("menu.font.size", menu_font_size);
+	asxml_var_insert(ASXMLVAR_MenuFontSize, menu_font_size);
 
 
     mystyle_list_set_property (Scr.wmprops, look->styles_list);
@@ -1170,17 +1325,6 @@ FixLook( MyLook *look )
         }
     }
 
-#ifdef LOCAL_DEBUG
-    LOCAL_DEBUG_OUT( "syncing %s","");
-    ASSync(False);
-#endif
-
-#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
-    Print_balloonConfig ( &BalloonConfig );
-#endif
-    balloon_config2look( look, &BalloonConfig, "TitleButtonBalloon" );
-    set_balloon_look( look->balloon_look );
-
     /* checking sanity of the move-resize window geometry :*/
     if( (look->resize_move_geometry.flags&(HeightValue|WidthValue)) != (HeightValue|WidthValue))
 	{
@@ -1263,6 +1407,7 @@ FixLook( MyLook *look )
 #endif
 }
 
+
 /*
  * Initialize database variables
  */
@@ -1295,7 +1440,7 @@ redecorate_aswindow_iter_func(void *data, void *aux_data)
 		redecorate_window( asw, False );
         if( asw->internal && asw->internal->on_look_feel_changed )
             asw->internal->on_look_feel_changed( asw->internal, &Scr.Feel, &Scr.Look, ASFLAGS_EVERYTHING );
-        on_window_status_changed( asw, True, True );
+        on_window_status_changed( asw, True );
 		set_flags( asw->internal_flags, ASWF_PendingShapeRemoval );
     }
     return True;
@@ -1512,11 +1657,15 @@ LoadASConfig (int thisdesktop, ASFlagType what)
             if( (const_configfile = get_session_file (Session, thisdesktop, F_CHANGE_LOOK, False) ) != NULL )
             {
                 InitLook (&Scr.Look, True);
+
 				memset( &TmpLook, 0x00, sizeof(TmpLook));
 				TmpLook.magic = MAGIC_MYLOOK ;
 				InitLook (&TmpLook, False );
+
 				LOCAL_DEBUG_OUT( "desk_anime_tint = %lX", TmpLook.desktop_animation_tint );
                 ParseConfigFile (const_configfile, &tline);
+
+
 				LOCAL_DEBUG_OUT( "desk_anime_tint = %lX", TmpLook.desktop_animation_tint );
                 show_progress("LOOK configuration loaded from \"%s\" ...", const_configfile);
                 display_progress( True, "LOOK configuration loaded from \"%s\".", const_configfile);
@@ -1536,6 +1685,7 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 				}
 #endif
 				merge_look( &Scr.Look, &TmpLook );
+			
             }else
             {
                 show_warning("LOOK configuration file cannot be found!");
@@ -1544,8 +1694,9 @@ LoadASConfig (int thisdesktop, ASFlagType what)
             }
         	if( UpdateGtkRC(Environment) ) 
 				ExecuteFunction (&gtkrc_signal_func, &dummy_event, -1);
-			if( UpdateKCSRC() ) 
-				ExecuteFunction (&kde_signal_func, &dummy_event, -1);
+			if( !get_flags(Environment->flags,ASE_NoKDEGlobalsTheming ) ) 
+				if( UpdateKCSRC() ) 
+					ExecuteFunction (&kde_signal_func, &dummy_event, -1);
 		}
         if (get_flags(what, PARSE_FEEL_CONFIG))
 		{
@@ -1597,23 +1748,6 @@ LoadASConfig (int thisdesktop, ASFlagType what)
                 clear_flags(what, PARSE_FEEL_CONFIG);
             }
         }
-        if (get_flags(what, PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
-		{
-            if( (const_configfile = get_session_file (Session, thisdesktop, F_CHANGE_THEME, False) ) != NULL )
-            {
-                ParseConfigFile (const_configfile, &tline);
-                show_progress("THEME configuration loaded from \"%s\" ...", const_configfile);
-                display_progress( True, "THEME configuration loaded from \"%s\" .", const_configfile);
-                if( (configfile = make_session_data_file  (Session, False, R_OK, THEME_OVERRIDE_FILE, NULL )) != NULL )
-                {
-                    ParseConfigFile (configfile, &tline);
-                    show_progress("THEME OVERRIDES configuration loaded from \"%s\" ...", configfile);
-                    display_progress( True, "THEME OVERRIDES configuration loaded from \"%s\" .", configfile);
-                    free( configfile );
-                }
-            }
-        }
-
         if (get_flags(what, PARSE_DATABASE_CONFIG))
 		{
 			if( !ReloadASDatabase() )
@@ -1630,7 +1764,9 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 	} else
 	{
 		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True, True );
+
 		LoadColorScheme();
+
 		memset( &TmpLook, 0x00, sizeof(TmpLook));
 		InitLook (&TmpLook, False );
 		InitLook (&Scr.Look, True);
@@ -1638,15 +1774,17 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 		InitFeel (&TmpFeel, False );
         InitFeel (&Scr.Feel, True);
         ParseConfigFile (Session->overriding_file, &tline);
-        ReloadASDatabase();
 		merge_look( &Scr.Look, &TmpLook );
 		merge_feel( &Scr.Feel, &TmpFeel );
+
+        ReloadASDatabase();
         show_progress("AfterStep configuration loaded from \"%s\" ...", Session->overriding_file);
         display_progress( True, "AfterStep configuration loaded from \"%s\".", Session->overriding_file);
         what = PARSE_EVERYTHING ;
 		if( UpdateGtkRC(Environment) ) 
 			ExecuteFunction (&gtkrc_signal_func, &dummy_event, -1);
-		UpdateKCSRC();
+		if( !get_flags(Environment->flags,ASE_NoKDEGlobalsTheming ) ) 
+			UpdateKCSRC();
     }
 
 	/* let's free the memory used for parsing */
@@ -1660,13 +1798,26 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 
     if (get_flags(what, PARSE_FEEL_CONFIG))
 	{
+	    display_progress( True, "Applying Feel.");
 		check_feel_sanity( &Scr.Feel );
         ApplyFeel( &Scr.Feel );
+		asxml_var_insert(ASXMLVAR_MenuRecentSubmenuItems, Scr.Feel.recent_submenu_items );
 	}
 
     if (get_flags(what, PARSE_LOOK_CONFIG))
     {
         FixLook( &Scr.Look );
+
+		asxml_var_insert(ASXMLVAR_IconButtonWidth, Scr.Look.ButtonWidth);
+		asxml_var_insert(ASXMLVAR_IconButtonHeight, Scr.Look.ButtonHeight);
+
+		asxml_var_insert(ASXMLVAR_MinipixmapWidth, Scr.Look.minipixmap_width);
+		asxml_var_insert(ASXMLVAR_MinipixmapHeight, Scr.Look.minipixmap_height);
+
+		asxml_var_insert(ASXMLVAR_MenuShowMinipixmaps, get_flags(Scr.Look.flags, MenuMiniPixmaps)?1:0);
+		asxml_var_insert(ASXMLVAR_MenuShowUnavailable, get_flags(Scr.Look.flags, MenuShowUnavailable)?1:0);
+		asxml_var_insert(ASXMLVAR_MenuTxtItemsInd,     get_flags(Scr.Look.flags, TxtrMenuItmInd)?1:0);
+
         if( thisdesktop == Scr.CurrentDesk )
 		{
 		    MyBackground *new_back = get_desk_back_or_default( Scr.CurrentDesk, False );
@@ -1679,10 +1830,16 @@ LoadASConfig (int thisdesktop, ASFlagType what)
             change_desktop_background( Scr.CurrentDesk );
 		}
     }
+	
+	if (get_flags(what, PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
+	{
+		ReloadConfig(what);
+    }
 
     if( get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
 	{
-        ASHashIterator  i;
+    	int count = 0 ;
+	    ASHashIterator  i;
 		ARGB32 cursor_fore = ARGB32_White ;
 		ARGB32 cursor_back = ARGB32_Black ;
 
@@ -1695,14 +1852,27 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 	   		parse_argb_color( Scr.Look.CursorBack, &cursor_back );
 		recolor_feel_cursors( &Scr.Feel, cursor_fore, cursor_back );
 		XDefineCursor (dpy, Scr.Root, Scr.Feel.cursors[ASCUR_Default]);
-        
+
+	    display_progress( True, "Reloading menu pixmaps :");
 		if( start_hash_iteration (Scr.Feel.Popups, &i) )
             do
             {
-				reload_menu_pmaps( (MenuData *)curr_hash_data(&i) );
+				MenuData *md = curr_hash_data(&i);
+				{
+					char *name = md->name;
+					Bool newline = ( count%10 == 0 );
+					if( isdigit(name[0]) )
+						if( md->first != NULL && md->first->fdata->func == F_TITLE ) 
+					    	name = md->first->item;
+					display_progress( newline, newline?"    %s":"%s", name);
+					++count;
+				}
+				reload_menu_pmaps( md, get_flags(what, PARSE_BASE_CONFIG) );
             }while( next_hash_item( &i ));
 		
+	    display_progress( True, "Advertising titlebar properties ...");
 		advertise_tbar_props();
+	    display_progress( False, "Done.");
 	}
 	   
     /* force update of window frames */
@@ -2341,4 +2511,5 @@ match_string (struct config *table, char *text, char *error_msg, FILE * fd)
 		tline_error (error_msg);
 }
 
+#endif
 
