@@ -268,7 +268,7 @@ check_frame_canvas( ASWindow *asw, Bool required )
 				bw = asw->hints->border_width ;
 			asw->status->frame_border_width = bw ;
             w = create_visual_window (Scr.asv, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, 
-									  0, 0, 
+									  asw->status->x, asw->status->y, 
 									  asw->status->width, asw->status->height,
                                       bw, InputOutput, valuemask, &attributes);
 			XLowerWindow( dpy, w );
@@ -368,6 +368,7 @@ LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw
         if( (asw->status == NULL || !get_flags(asw->status->flags, AS_Dead)) && 
 			get_parent_window( w ) == asw->frame )
         {
+			LOCAL_DEBUG_OUT( "reparenting client window %lX", w );
             if( get_flags( AfterStepState, ASS_Shutdown ) )
                 quietly_reparent_window( w, Scr.Root, xwc.x, xwc.y, AS_CLIENT_EVENT_MASK );
             else
@@ -406,7 +407,7 @@ check_icon_canvas( ASWindow *asw, Bool required )
                  !get_flags(Scr.Feel.flags, KeepIconWindows))
             { /* create windows */
                 attributes.event_mask = AS_ICON_TITLE_EVENT_MASK;
-                w = create_visual_window ( Scr.asv, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, -10, -10, 1, 1, 0,
+                w = create_visual_window ( Scr.asv, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, -9000, -9000, 1, 1, 0,
                                            InputOutput, valuemask, &attributes );
                 canvas = create_ascanvas( w );
             }else
@@ -467,7 +468,7 @@ check_icon_title_canvas( ASWindow *asw, Bool required, Bool reuse_icon_canvas )
             attributes.cursor = Scr.Feel.cursors[ASCUR_Default];
             attributes.event_mask = AS_ICON_TITLE_EVENT_MASK;
             /* create windows */
-            w = create_visual_window ( Scr.asv, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, -10, -10, 1, 1, 0,
+            w = create_visual_window ( Scr.asv, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, -9010, -9010, 1, 1, 0,
                                        InputOutput, valuemask, &attributes );
 
             register_aswindow( w, asw );
@@ -1234,7 +1235,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
       		    {
           		    char *str = list_functions_by_context (C_TButton0<<i, asw->hints);
               		LOCAL_DEBUG_OUT( "balloon text will be \"%s\"", str?str:"none" );
-	                set_astbar_balloon( asw->tbar, C_TButton0<<i, str, AS_Text_ASCII );
+	                set_astbar_balloon2( asw->tbar, TitlebarBalloons, C_TButton0<<i, str, AS_Text_ASCII );
   		            if( str )
       		            free( str );
           		}
@@ -1257,8 +1258,11 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 		/* we need to map all the possibly created subwindows if window is not iconic */
 		if( !ASWIN_GET_FLAGS(asw, AS_Iconic ) )
 		{
-			LOCAL_DEBUG_OUT("mapping frame subwindows for client %lX, frame canvas = %p", asw->w, asw->frame_canvas );
-			XMapSubwindows(dpy, asw->frame);
+			for( i = 0 ; i < FRAME_SIDES ; ++i )
+				map_canvas_window( asw->frame_sides[i], False );
+
+/*			LOCAL_DEBUG_OUT("mapping frame subwindows for client %lX, frame canvas = %p", asw->w, asw->frame_canvas );
+			XMapSubwindows(dpy, asw->frame); */
 		}else
 		{
 			map_canvas_window( asw->icon_canvas, False );
@@ -1387,7 +1391,9 @@ SetShape (ASWindow *asw, int w)
 			/* add clients shape */
 			combine_canvas_shape_at_geom (asw->frame_canvas, asw->client_canvas, child_x, child_y, width, height, bw );
 
+#ifndef NO_DEBUG_OUTPUT
 			print_shape( asw->frame_canvas->shape ) ;
+#endif
 			/* apply shape */
 			update_canvas_display_mask (asw->frame_canvas, True);
 			set_flags( asw->internal_flags, ASWF_PendingShapeRemoval );
