@@ -338,19 +338,62 @@ DigestEvent( ASEvent *event )
 {
     register int i ;
     setup_asevent_from_xevent( event );
+    event->client = NULL;
+    SHOW_EVENT_TRACE(event);
     /* in housekeeping mode we handle pointer events only as applied to root window ! */
     if( Scr.moveresize_in_progress && (event->eclass & ASE_POINTER_EVENTS) != 0)
     {
 	    event->context = C_ROOT ;
     	event->widget = Scr.RootCanvas ;
-        event->client = NULL;
         /* we have to do this at all times !!!! */
         if( event->x.type == ButtonRelease && Scr.Windows->pressed )
             release_pressure();
     }else
 	{
-		if( event->w == Scr.Root || 
-			event->w == Scr.ServiceWin ||
+	    XButtonEvent *xbtn = &(event->x.xbutton);
+
+		if( event->w == Scr.Root )
+		{
+			event->context = C_ROOT ;
+#ifndef NO_VIRTUAL
+			if((event->eclass & ASE_MousePressEvent) != 0 && xbtn->subwindow != None )
+			{
+				LOCAL_DEBUG_OUT( "subwindow = %lX", event->x.xbutton.subwindow );
+	    		for( i = 0 ; i < PAN_FRAME_SIDES ; i++ )
+	    		{
+	        		LOCAL_DEBUG_OUT("checking panframe %d, mapped %d", i, Scr.PanFrame[i].isMapped );
+	        		if( Scr.PanFrame[i].isMapped && xbtn->subwindow == Scr.PanFrame[i].win)
+        			{/* we should try and pass through this click onto any client that 
+						maybe under the pan frames */
+						LOCAL_DEBUG_OUT( "looking for client at %+d%+d", xbtn->x_root, xbtn->y_root ); 
+						if( (event->client = find_topmost_client( Scr.CurrentDesk, xbtn->x_root, xbtn->y_root )) != NULL )
+						{
+					    	LOCAL_DEBUG_OUT("underlying new client %p", event->client );
+							ASCanvas *cc = event->client->client_canvas ; 
+							if( cc && cc->root_x <= xbtn->x_root && cc->root_y <= xbtn->y_root &&
+								cc->root_x+cc->width > xbtn->x_root && cc->root_y+cc->height > xbtn->y_root )
+							{
+								event->w = xbtn->window = event->client->w ; 
+								xbtn->subwindow = None ; 
+						        XSendEvent (dpy, event->client->w, False, 
+											(xbtn->type == ButtonPress)?ButtonPressMask:ButtonReleaseMask, &(event->x));
+								xbtn->subwindow = event->client->w ; 
+								xbtn->window = Scr.Root ;
+								/* xbtn->window should still be Root, so we can proxy it down to the app */
+							}else
+							{
+								event->w = xbtn->window = event->client->frame ; 
+								xbtn->subwindow = None ;
+							}
+					    	LOCAL_DEBUG_OUT("new event window %lX", event->w );
+							event->context = C_NO_CONTEXT ;						
+						}
+						break;
+					}
+				}
+			}
+#endif
+		}else if( event->w == Scr.ServiceWin ||
 			event->w == Scr.SizeWindow )
 		{
 	    	event->context = C_ROOT ; 
@@ -361,9 +404,9 @@ DigestEvent( ASEvent *event )
 		{	
     		event->widget = Scr.RootCanvas ;
 			event->client = NULL ;
-		}else
+		}else if(event->client == NULL )
 		{
-			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) != NULL  )
 			{	
 				event->client = NULL ;
 				event->widget = NULL ;	 
@@ -501,6 +544,10 @@ DigestEvent( ASEvent *event )
 								  	(event->x.type == LeaveNotify),
 								  	(last_pointer_root_x != pointer_root_x || last_pointer_root_y != pointer_root_y) );
 		}		
+		if( event->x.type == LeaveNotify ) 
+		{
+			withdraw_active_balloon_from( TitlebarBalloons );
+		}
 		last_pointer_root_x = pointer_root_x ;
 		last_pointer_root_y = pointer_root_y ;
 
@@ -718,9 +765,12 @@ HandleFocusIn ( ASEvent *event )
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         ChangeWarpingFocus( event->client );
 
+	LOCAL_DEBUG_OUT( "focused = %p, this event for %p", Scr.Windows->focused, event->client );
     if( Scr.Windows->focused != event->client )
+	{
+		LOCAL_DEBUG_OUT( "CHANGE Scr.Windows->focused from %p to NULL", Scr.Windows->focused );
         Scr.Windows->focused = NULL ;
-
+	}
     if( event->client == NULL && get_flags(AfterStepState, ASS_HousekeepingMode))
         return;
     /* note that hilite_aswindow changes value of Scr.Hilite!!! */
@@ -1018,13 +1068,22 @@ HandleDestroyNotify (ASEvent *event )
  *  Procedure:
  *	HandleMapRequest - MapRequest event handler
  ************************************************************************/
+void
+delayed_add_window( void* vdata )
+{
+	if( window2ASWindow((Window)vdata) == NULL )
+		AddWindow ((Window)vdata, True);
+}
+
 void
 HandleMapRequest (ASEvent *event )
 {
     /* If the window has never been mapped before ... */
     if (event->client == NULL)
-    {
-        if( (event->client = AddWindow (event->w, True)) == NULL )
+    {/* lets delay handling map request in case client needs time to update its properties */
+		timer_new (200, delayed_add_window, (void*)event->w);	
+
+/*        if( (event->client = AddWindow (event->w, True)) == NULL ) */
             return;
     }else /* If no hints, or currently an icon, just "deiconify" */
         set_window_wm_state( event->client, False, True );
@@ -1080,7 +1139,7 @@ HandleMapNotify ( ASEvent *event )
     ASWIN_CLEAR_FLAGS(asw, AS_Iconic);
     complete_wm_state_transition( asw, NormalState );
 	LOCAL_DEBUG_OUT( "no_focus = %d, force_activation = %d, AcceptsFocus = %ld", no_focus, force_activation, ASWIN_HFLAGS(asw,AS_AcceptsFocus) );
-	if( !no_focus && ASWIN_HFLAGS(asw,AS_AcceptsFocus) )
+	if( !no_focus && ASWIN_FOCUSABLE(asw) )
     	activate_aswindow (asw, force_activation, False);
     broadcast_config( M_MAP, asw );
     /* finally reaches Normal state */
@@ -1310,7 +1369,7 @@ HandleEnterNotify (ASEvent *event)
     if (asw == NULL )
 		return;
 
-    if (ASWIN_HFLAGS(asw,AS_AcceptsFocus))
+    if (ASWIN_FOCUSABLE(asw))
 	{
         if (!get_flags(Scr.Feel.flags, ClickToFocus) || asw->internal != NULL )
 		{
@@ -1364,6 +1423,7 @@ HandleConfigureRequest ( ASEvent *event )
     XConfigureRequestEvent *cre = &(event->x.xconfigurerequest);
     ASWindow *asw = event->client ;
     XWindowChanges xwc;
+    unsigned long xwcm;
 
     /*
 	 * According to the July 27, 1988 ICCCM draft, we should ignore size and
@@ -1376,29 +1436,67 @@ HandleConfigureRequest ( ASEvent *event )
 	cre->x, cre->y );
     if (asw == NULL)
 	{
-        unsigned long xwcm;
         xwcm = cre->value_mask & (CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
-		xwc.x = cre->x;
-		xwc.y = cre->y;
-		xwc.width = cre->width;
-		xwc.height = cre->height;
-		xwc.border_width = cre->border_width;
-		LOCAL_DEBUG_OUT( "Configuring untracked window %lX to %dx%d%+d%+d and bw = %d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, cre->x, cre->y, cre->border_width, xwcm );
-        XConfigureWindow (dpy, event->w, xwcm, &xwc);
+		if( xwcm == 0 ) 
+		{
+			LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for untracked window %lX - no supported changes detected.", (unsigned long)event->w );
+		}else
+		{
+			xwc.x = cre->x;
+			xwc.y = cre->y;
+			xwc.width = cre->width;
+			xwc.height = cre->height;
+			xwc.border_width = cre->border_width;
+			LOCAL_DEBUG_OUT( "Configuring untracked window %lX to %dx%d%+d%+d and bw = %d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, cre->x, cre->y, cre->border_width, xwcm );
+        	XConfigureWindow (dpy, event->w, xwcm, &xwc);
+			ASSync( False);
+		}
 		return;
 	}
 
-	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) ||
+	if( asw->icon_canvas && event->w == asw->icon_canvas->w ) 
+	{ /* we really should ignore that ! - let's see how it will play out */
+	  /* we may need to add code to iconbox to handle custom icon geometry for client icons */
+        xwcm = cre->value_mask & (CWWidth | CWHeight);
+		if( xwcm == 0 ) 
+		{
+			LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for iconic window %lX - no supported changes detected.", (unsigned long)event->w );
+		}else
+		{
+			xwc.width = cre->width;
+			xwc.height = cre->height;
+			LOCAL_DEBUG_OUT( "Configuring iconic window %lX to %dx%d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, xwcm );
+        	XConfigureWindow (dpy, event->w, xwcm, &xwc);
+			ASSync( False);
+		}
+		return;
+	}
+	
+	if( event->w != asw->w ) 
+	{
+		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for window %lX. Not a client or client's icon!", (unsigned long)event->w );
+		return;
+	}
+
+    if (cre->value_mask & CWStackMode )
+	{
+		if( !ASWIN_HFLAGS(asw, AS_IgnoreRestackRequest ) )
+		{
+	        restack_window( asw, (cre->value_mask & CWSibling)?cre->above:None, cre->detail );
+		}else
+		{
+			LOCAL_DEBUG_OUT( "Ignoring Stacking order Request for client %p as required by hints", asw );
+		}
+	}
+
+	if( ( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) && !get_flags(cre->value_mask, CWWidth|CWHeight))||
 		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
 	{	
 		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
-		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas);
+		SendConfigureNotify(asw);
 		return;
 	}
 
-    if (cre->value_mask & CWStackMode)
-        restack_window( asw, (cre->value_mask & CWSibling)?cre->above:None, cre->detail );
-
     /* check_aswindow_shaped( asw ); */
 
     /* for restoring */
@@ -1410,25 +1508,28 @@ HandleConfigureRequest ( ASEvent *event )
     if( cre->value_mask & (CWWidth|CWHeight|CWX|CWY) )
     {
         XRectangle new_anchor = asw->anchor;
-        int grav_x, grav_y ;
-        get_gravity_offsets (asw->hints, &grav_x, &grav_y);
-
 
         if( cre->value_mask&CWWidth )
             new_anchor.width = cre->width ;
         if( cre->value_mask&CWHeight )
             new_anchor.height = cre->height ;
-        if( cre->value_mask&CWX )
-            new_anchor.x = make_anchor_pos ( asw->status, cre->x, new_anchor.width, Scr.Vx, grav_x, Scr.VxMax+Scr.MyDisplayWidth );
-        if( cre->value_mask&CWY )
-            new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
-
+		if( !ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+		{
+    	    int grav_x, grav_y ;
+	        get_gravity_offsets (asw->hints, &grav_x, &grav_y);
+        	if( cre->value_mask&CWX )
+            	new_anchor.x = make_anchor_pos ( asw->status, cre->x, new_anchor.width, Scr.Vx, grav_x, Scr.VxMax+Scr.MyDisplayWidth );
+        	if( cre->value_mask&CWY )
+            	new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
+		}
 LOCAL_DEBUG_OUT( "old anchor(%dx%d%+d%+d), new_anchor(%dx%d%+d%+d)", asw->anchor.width, asw->anchor.height, asw->anchor.x, asw->anchor.y, new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
         validate_window_anchor( asw, &new_anchor );
 LOCAL_DEBUG_OUT( "validated_anchor(%dx%d%+d%+d)", new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
         asw->anchor = new_anchor ;
         on_window_anchor_changed( asw );
 		enforce_avoid_cover( asw );
+        if( (cre->value_mask&(CWWidth|CWHeight)) == 0 )
+			SendConfigureNotify(asw);		
     }
 }
 
@@ -1579,12 +1680,13 @@ afterstep_wait_pipes_input(int timeout_sec)
         i = MIN(MODULES_NUM,Module_npipes) ;
         /* Check for module input. */
         while( --i >= 0 )
-        {
-            Bool has_input = FD_ISSET (list[i].fd, &in_fdset);
-            Bool has_output = FD_ISSET (list[i].fd, &out_fdset);
-            if( has_input || has_output )
-                HandleModuleInOut(i, has_input, has_output);
-        }
+			if( list[i].fd > 0 )
+	        {
+        	    Bool has_input = FD_ISSET (list[i].fd, &in_fdset);
+    	        Bool has_output = FD_ISSET (list[i].fd, &out_fdset);
+            	if( has_input || has_output )
+                	HandleModuleInOut(i, has_input, has_output);
+	        }
 	}
 
 	/* handle timeout events */
