@@ -69,6 +69,7 @@ void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module );
 void close_func_handler( FunctionData *data, ASEvent *event, int module );
 void restart_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_func_handler( FunctionData *data, ASEvent *event, int module );
+void desktop_entry_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_background_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -96,6 +97,7 @@ void send_window_list_func_handler( FunctionData *data, ASEvent *event, int modu
 void save_workspace_func_handler( FunctionData *data, ASEvent *event, int module );
 void signal_reload_GTKRC_file_handler( FunctionData *data, ASEvent *event, int module );
 void KIPC_send_message_all_handler( FunctionData *data, ASEvent *event, int module );
+void set_func_handler( FunctionData *data, ASEvent *event, int module );
 void test_func_handler( FunctionData *data, ASEvent *event, int module );
 void screenshot_func_handler( FunctionData *data, ASEvent *event, int module );
 void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -149,6 +151,8 @@ void SetupFunctionHandlers()
 	function_handlers[F_CLOSE]          	= close_func_handler ;
     function_handlers[F_RESTART]            = restart_func_handler ;
 
+    function_handlers[F_DesktopEntry]		= desktop_entry_func_handler;
+
     function_handlers[F_EXEC] 				=
 		function_handlers[F_Swallow] 		=
 		function_handlers[F_MaxSwallow] 	= exec_func_handler ;
@@ -211,6 +215,7 @@ void SetupFunctionHandlers()
 
     function_handlers[F_QUICKRESTART]       = quickrestart_func_handler ;
     function_handlers[F_SEND_WINDOW_LIST]   = send_window_list_func_handler ;
+    function_handlers[F_SET]                = set_func_handler ;
     function_handlers[F_Test]               = test_func_handler ;
 
 	function_handlers[F_TAKE_WINDOWSHOT]    =
@@ -258,6 +263,7 @@ void destroy_scheduled_function_handler( void *data )
 }
 #define destroy_scheduled_function(sf)  destroy_scheduled_function_handler((void*)sf)
 
+static void DoExecuteFunction ( ASScheduledFunction *sf );
 
 /***********************************************************************
  *  Procedure:
@@ -301,7 +307,11 @@ ExecuteFunctionExt (FunctionData *data, ASEvent *event, int module, Bool defered
 	if( event->widget )
 	    sf->canvas              = event->widget->w ;
     }
-    append_bidirelem( FunctionQueue, sf );
+	/* we may end up deadlocked if we wait for modules who wait for window list : */
+	if( data->func == F_SEND_WINDOW_LIST ) 
+		DoExecuteFunction ( sf );
+	else
+	    append_bidirelem( FunctionQueue, sf );
 }
 
 void
@@ -750,14 +760,9 @@ Bool desktop_category2complex_function( const char *name, const char *category_n
 		++(func->items_num);
 		init_func_data( fdata );
 		fdata->name = mystrdup("I");  /* for immidiate execution */ 
-		
-		if( get_flags( de->flags, ASDE_Terminal ) )
-			fdata->func = F_ExecInTerm;	   
-		else if( get_flags( de->flags, ASDE_ASModule ) )
-			fdata->func = F_MODULE;	   
-		else
-		 	fdata->func = F_EXEC;	
-        fdata->text = mystrdup( de->clean_exec );
+	 	fdata->func = desktop_entry2function_code( de );	
+		if( de->clean_exec ) 
+	        fdata->text = mystrdup( de->clean_exec );
 	}	
 
 	return True;	   
@@ -797,9 +802,17 @@ void moveresize_func_handler( FunctionData *data, ASEvent *event, int module )
 	if( data->func == F_MOVE )
 	{
 	    if( data->func_val[0] != INVALID_POSITION )
-		x = new_val1;
+		{
+			x = new_val1;
+			if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
+				x -= asw->status->viewport_x ;
+		}
 	    if( data->func_val[1] != INVALID_POSITION )
-		y = new_val2;
+		{
+			y = new_val2;
+			if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
+				y -= asw->status->viewport_y ;
+		}
 	}else
 	{
 	    if( data->func_val[0] != INVALID_POSITION )
@@ -808,66 +821,64 @@ void moveresize_func_handler( FunctionData *data, ASEvent *event, int module )
 		height = new_val2;
 	}
 	moveresize_aswindow_wm( asw, x, y, width, height, False );
+
     }else
     {
-	ASMoveResizeData *mvrdata;
-	/*release_pressure(); */
-	if( data->func == F_MOVE )
-	    mvrdata = move_widget_interactively(Scr.RootCanvas,
-						asw->frame_canvas,
-						event,
-						apply_aswindow_move,
-						complete_aswindow_move );
-	else
-	{
-	    int side = 0 ;
-	    register unsigned long context = (event->context&C_FRAME);
+		ASMoveResizeData *mvrdata;
+		/*release_pressure(); */
+		if( data->func == F_MOVE )
+		{
+	    	mvrdata = move_widget_interactively(Scr.RootCanvas,
+							asw->frame_canvas,
+							event,
+							apply_aswindow_move,
+							complete_aswindow_move );
+		}else
+		{
+	    	int side = 0 ;
+	    	register unsigned long context = (event->context&C_FRAME);
 
-	    if( ASWIN_GET_FLAGS( asw, AS_Shaded ) )
-	    {
-		XBell (dpy, Scr.screen);
-		return;
-	    }
+	    	if( ASWIN_GET_FLAGS( asw, AS_Shaded ) )
+	    	{
+			XBell (dpy, Scr.screen);
+			return;
+	    	}
 
-	    while( (0x01&context) == 0 && side <= FR_SE)
-	    {
-		++side ;
-		context = context>>1 ;
-	    }
+	    	while( (0x01&context) == 0 && side <= FR_SE)
+	    	{
+			++side ;
+			context = context>>1 ;
+	    	}
 
-			if( side > FR_SE )
-			{
-				int pointer_x = 0, pointer_y = 0 ;
-				ASQueryPointerRootXY( &pointer_x, &pointer_y );
-				if( pointer_x > asw->frame_canvas->root_x + asw->frame_canvas->width/2 )
+				if( side > FR_SE )
 				{
-					if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
-						side = FR_SE ;
+					int pointer_x = 0, pointer_y = 0 ;
+					ASQueryPointerRootXY( &pointer_x, &pointer_y );
+					if( pointer_x > asw->frame_canvas->root_x + asw->frame_canvas->width/2 )
+					{
+						if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
+							side = FR_SE ;
+						else
+							side = FR_NE ;
+					}else if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
+						side = FR_SW ;
 					else
-						side = FR_NE ;
-				}else if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
-					side = FR_SW ;
-				else
-					side = FR_NW ;
-			}
+						side = FR_NW ;
+				}
 
-	    mvrdata = resize_widget_interactively(  Scr.RootCanvas,
-						    asw->frame_canvas,
-						    event,
-						    apply_aswindow_moveresize,
-						    complete_aswindow_moveresize,
-						    side );
-	}
-	if( mvrdata )
-	{
+	    	mvrdata = resize_widget_interactively(  Scr.RootCanvas,
+						    	asw->frame_canvas,
+						    	event,
+						    	apply_aswindow_moveresize,
+						    	complete_aswindow_moveresize,
+						    	side );
+		}
+		if( mvrdata )
+		{
 			mvrdata->move_only = (data->func == F_MOVE) ;
-	    raise_scren_panframes( ASDefaultScr );
-	    mvrdata->below_sibling = get_lowest_panframe(ASDefaultScr);
-	    set_moveresize_restrains( mvrdata, asw->hints, asw->status);
-	    mvrdata->grid = make_desktop_grid(Scr.CurrentDesk, AS_LayerDesktop, False, Scr.Vx, Scr.Vy, asw);
-	    Scr.moveresize_in_progress = mvrdata ;
-	    ASWIN_SET_FLAGS( asw, AS_MoveresizeInProgress );
-	}
+			setup_aswindow_moveresize(asw, mvrdata );
+	    	ASWIN_SET_FLAGS( asw, AS_MoveresizeInProgress );
+		}
     }
 }
 
@@ -1091,11 +1102,13 @@ void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     ASMenu *menu = NULL;
 	char * menu_name = data->text? data->text:data->name ;
-    if( menu_name )
+	LOCAL_DEBUG_OUT( "menu_name = \"%s\", client = %p, internal = %p", menu_name?"NULL":menu_name, event->client, event->client->internal );
+    if( menu_name && menu_name[0] != '\0' )
 		menu = find_asmenu( menu_name );
     else if( event->client && event->client->internal )
     {
 		ASMagic *data = event->client->internal->data ;
+		LOCAL_DEBUG_OUT( "data = %p, magic = %lX", data, data->magic );
 		if( data->magic == MAGIC_ASMENU )
 	    	menu = (ASMenu*)data;
     }
@@ -1282,6 +1295,23 @@ void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module )
 	}
 }
 
+void desktop_entry_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	ASDesktopEntry *de = name2desktop_entry( data->text?data->text:data->name, NULL );
+	if( de == NULL ) 
+		XBell (dpy, event->scr->screen);
+	else
+	{
+		FunctionData *real_data = desktop_entry2function( de, NULL);
+		if( real_data ) 
+		{
+			ExecuteFunctionExt( real_data, event, -1, True );
+			destroy_func_data( &real_data );
+		}
+	}	
+}
+
+
 static int _as_config_change_recursion = 0 ;
 static int _as_config_change_count = 0 ;
 static int _as_background_change_count = 0 ;
@@ -1658,6 +1688,8 @@ void refresh_func_handler( FunctionData *data, ASEvent *event, int module )
     XSync (dpy, False);
     XDestroyWindow (dpy, w);
     XFlush (dpy);
+	
+	spool_unfreed_mem( "afterstep.allocs.refresh", NULL );
 }
 
 void goto_page_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1859,12 +1891,11 @@ void restartmodule_func_handler( FunctionData *data, ASEvent *event, int module
 {
 	char *cmd_line = GetModuleCmdLineByName(data->text);
 	KillModuleByName (data->text);	
-	if( cmd_line ) 
-	{
-	    UngrabEm ();
-    	ExecModule (cmd_line, event->client ? event->client->w : None, event->context);
+	
+    UngrabEm ();
+   	ExecModule (cmd_line?cmd_line:data->text, event->client ? event->client->w : None, event->context);
+	if( cmd_line )
 		free( cmd_line );
-	}	 
 }
 
 void popup_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1883,8 +1914,14 @@ void windowlist_func_handler( FunctionData *data, ASEvent *event, int module )
     MenuData *md =  make_desk_winlist_menu( Scr.Windows, data->text == NULL ? event->scr->CurrentDesk: data->func_val[0], Scr.Feel.winlist_sort_order, False );
     if( md != NULL )
 	{	
-		run_menu_data( md );
-		destroy_menu_data( &md );
+		ASMenu *menu = run_menu_data( md );
+			/* attaching menu data to menu, so that we can destroy it when menu closes.
+			   This is to accomodate those dynamically generated menus, 
+			   such as window list and module list */
+		   	/* Crude! should implement reference counting instead  */
+		
+		if( menu ) 
+			menu->volitile_menu_data = md ; 
 	}	
 #endif /* ! NO_WINDOWLIST */
 }
@@ -1896,8 +1933,9 @@ void modulelist_func_handler( FunctionData *data, ASEvent *event, int module )
 						make_restart_module_menu(Scr.Feel.winlist_sort_order );
     if( md != NULL )
 	{	
-		run_menu_data( md );
-		destroy_menu_data( &md );
+		ASMenu *menu = run_menu_data( md );
+		if( menu ) 
+			menu->volitile_menu_data = md ; 
 	}	 
 
 }
@@ -1930,12 +1968,12 @@ void send_window_list_func_handler( FunctionData *data, ASEvent *event, int modu
     {
 		union { void *ptr ; int id; } module_id;
 		module_id.id = module ;
-	SendPacket (module, M_TOGGLE_PAGING, 1, DoHandlePageing);
-	SendPacket (module, M_NEW_DESKVIEWPORT, 3, Scr.Vx, Scr.Vy, Scr.CurrentDesk);
-	iterate_asbidirlist( Scr.Windows->clients, send_aswindow_data_iter_func, module_id.ptr, NULL, False );
-	SendPacket (module, M_END_WINDOWLIST, 0);
-	if( IsValidDesk(Scr.CurrentDesk) )
-	    restack_window_list( Scr.CurrentDesk, True );
+		SendPacket (module, M_TOGGLE_PAGING, 1, DoHandlePageing);
+		SendPacket (module, M_NEW_DESKVIEWPORT, 3, Scr.Vx, Scr.Vy, Scr.CurrentDesk);
+		iterate_asbidirlist( Scr.Windows->clients, send_aswindow_data_iter_func, module_id.ptr, NULL, False );
+		SendPacket (module, M_END_WINDOWLIST, 0);
+		if( IsValidDesk(Scr.CurrentDesk) )
+			send_stacking_order( Scr.CurrentDesk );
     }
 }
 
@@ -2001,6 +2039,54 @@ void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module
  	}	
 }	 
 
+void set_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	if( data->text != NULL ) 
+	{
+		char *eq_ptr = strchr( data->text,'=' ); 
+		if( eq_ptr ) 
+		{
+			int val  = 0 ;
+			char *tail = eq_ptr+1; 
+			val = (int)parse_math(tail, &tail, strlen(tail));
+			LOCAL_DEBUG_OUT( "tail = \"%s\", val = %d", tail, val );
+			if( tail > eq_ptr+1 ) 
+			{
+				*eq_ptr = '\0' ; 
+				asxml_var_insert(data->text, val);
+				if( strncmp( data->text, "menu.", 5 ) == 0 )
+				{
+					ASFlagType change_flag = 0 ; 
+					ASFlagType old_look_flags = Scr.Look.flags ; 
+					if( strcmp( data->text, ASXMLVAR_MenuShowMinipixmaps ) == 0 )
+						change_flag = MenuMiniPixmaps ;
+					else if( strcmp( data->text, ASXMLVAR_MenuShowUnavailable ) == 0 )
+						change_flag = MenuShowUnavailable ;
+					else if( strcmp( data->text, ASXMLVAR_MenuTxtItemsInd ) == 0 )
+						change_flag = TxtrMenuItmInd ;
+					if( change_flag != 0 ) 
+					{
+						if( !val ) 
+							clear_flags( Scr.Look.flags, change_flag );
+						else 
+							set_flags( Scr.Look.flags, change_flag );
+					}else if( strcmp( data->text, ASXMLVAR_MenuRecentSubmenuItems) == 0 )
+					{
+						Scr.Feel.recent_submenu_items = (val <= 0)?0:val ;
+					}
+					
+					if( old_look_flags != Scr.Look.flags ) 
+					{
+						/* need to referesh menus maybe? */
+					}
+					LOCAL_DEBUG_OUT( "old_flags = %lX, flags = %lX", old_look_flags, Scr.Look.flags );
+				}
+				*eq_ptr = '=' ; 
+			}
+		}
+	}
+}
+
 void test_func_handler( FunctionData *data, ASEvent *event, int module )
 {
 	 /* add test command processing here : */
