@@ -130,8 +130,6 @@ LOCAL_DEBUG_OUT( "module name \"%s\"", module->name );
         	free_func_data (module->ibuf.func);
         	free (module->ibuf.func);
 		}
-		
-
     	memset( module, 0x00, sizeof(module_t) );
 	}else
 	{
@@ -555,7 +553,7 @@ int
 AcceptModuleConnection (int socket_fd)
 {
 	int           fd;
-	int           len = sizeof (struct sockaddr_un);
+	unsigned int  len = sizeof (struct sockaddr_un);
 	struct sockaddr_un name;
 
 	fd = accept (socket_fd, (struct sockaddr *)&name, &len);
@@ -697,7 +695,8 @@ LOCAL_DEBUG_OUT( "module %d has %s input and %s output", channel, has_input?"":"
             else if ( res > 0 ) /* need to run command */
                 RunCommand (module->ibuf.func, channel, module->ibuf.window);
         }
-        if( has_output )
+		/* module could be killed inside RunCoimmand !*/
+        if( has_output && module->fd > 0 )
             FlushQueue( module );
 	}
 }
@@ -712,7 +711,7 @@ DeadPipe (int nonsense)
 int FindModuleByName (char *name)
 {
 	int module = -1;
-	if( Modules ) 
+	if( Modules && name) 
 	{	
     	wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
 
@@ -746,8 +745,6 @@ GetModuleCmdLineByName(char *name)
 		register module_t *list = MODULES_LIST ;	
 		if( list[module].cmd_line ) 
 			return mystrdup( list[module].cmd_line ) ;
-		else
-			return mystrdup( list[module].name );
 	}
 	return NULL ;
 }
@@ -804,6 +801,7 @@ SendConfig (int module, send_data_type event_type, ASWindow * t)
     send_signed_data_type 	frame_x = 0, frame_y = 0, frame_width = 0, frame_height = 0;
     send_ID_type 			icon_title_w = None, icon_pixmap_w = None ;
     send_signed_data_type 	icon_x = 0, icon_y = 0, icon_width = 0, icon_height = 0 ;
+	send_signed_data_type 	pid = -1 ; 
 	union {
 		ASWindow *asw ;
 		send_data_type id ;
@@ -839,15 +837,19 @@ SendConfig (int module, send_data_type event_type, ASWindow * t)
         }
     }
 
+    if (ASWIN_HFLAGS(t,AS_PID))
+		pid = t->hints->pid ; 
+			
 	asw_id.asw = t ;
-    SendPacket (module, event_type, 26,
+    SendPacket (module, event_type, 27,
                 t->w,                 t->frame,              asw_id.id,
                 frame_x,              frame_y,               frame_width,         frame_height,
                 ASWIN_DESK(t),        t->status->flags,      t->hints->flags,     t->hints->client_icon_flags,
                 t->hints->base_width, t->hints->base_height, t->hints->width_inc, t->hints->height_inc,
                 t->hints->min_width,  t->hints->min_height,  t->hints->max_width,  t->hints->max_height,
                 t->hints->gravity,    icon_title_w,          icon_pixmap_w,
-                icon_x,               icon_y,                icon_width,           icon_height );
+                icon_x,               icon_y,                icon_width,           icon_height, 
+				pid );
 }
 
 void
@@ -915,6 +917,25 @@ SendStackingOrder (int channel, send_data_type msg_type, send_data_type desk, AS
     SendBuffer( channel );
 }
 
+static void 
+check_module_name_collision( unsigned int channel, const char *name, Bool kill_new )
+{
+	register int i = MODULES_NUM;
+    register module_t *list = MODULES_LIST ;
+LOCAL_DEBUG_OUT( "pid(%d),total modules %d. name = \"%s\", kill_new = %d", getpid(), MODULES_NUM, name, kill_new );
+    while(--i >= 0 )
+		if( i != channel ) 
+		{
+			LOCAL_DEBUG_OUT( "checking module %d, name = \"%s\"", i, list[i].name?list[i].name:"(null)" );
+            if( mystrcasecmp( list[i].name, name ) == 0 )
+			{
+		        KillModule (&(list[kill_new?channel:i]), False);
+				break;
+			}	
+		}
+}
+
+
 /* this will run command received from module */
 void
 RunCommand (FunctionData * fdata, unsigned int channel, Window w)
@@ -945,6 +966,8 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
 	        set_string( &(module->cmd_line), fdata->text );
     	    fdata->text = NULL;
 		}
+		if( Environment->module_name_collision != ASE_AllowModuleNameCollision ) 
+			check_module_name_collision( channel, module->name, (Environment->module_name_collision == ASE_KillNewModuleOnNameCollision) );			
         break;
      case F_UNLOCK:
 		 toret = 66;
@@ -1145,7 +1168,20 @@ make_module_menu( FunctionCode func, const char *title, int sort_order )
 				minipixmap = de->fulliconname ;
 
 			if( (mdi = add_menu_fdata_item( md, menuitems[i], minipixmap, NULL)) != NULL ) 
+			{
 				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+				if( de )
+				{
+					char *comment = NULL; 
+					if( dup_desktop_entry_Comment( de, &comment ) )
+						set_flags( mdi->flags, MD_CommentIsUTF8 );
+					if( comment )
+					{
+						mdi->comment = interpret_ascii_string( comment );
+						free( comment );
+					}
+				}
+			}
 			safefree( menuitems[i] ); /* scrubba-dub-dub */
 		}
 		safefree(menuitems);
